from plone import api
from Products.CMFPlone.resources.browser.cook import cookWhenChangingSettings


PROFILE_ID = 'profile-castle.cms:default'


def cook_js_resources(context, logger=None):
    cookWhenChangingSettings(api.portal.get())



from Products.CMFCore.utils import getToolByName
from plone import api
from Products.CMFPlone import PloneMessageFactory as _
from plone.app.users.browser.passwordpanel import PasswordPanel
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
import six
from DateTime import DateTime


class ChangePassword(PasswordPanel):

    @button.buttonAndHandler(
        _(u'label_change_password', default=u'Change Password'),
        name='reset_passwd'
    )
    def action_reset_passwd(self, action):
        data, errors = self.extractData()

        # extra password validation
        self.validate_password(action, data)

        if action.form.widgets.errors:
            self.status = self.formErrorsMessage
            return

        membertool = getToolByName(self.context, 'portal_membership')

        password = data['new_password']
        if six.PY2 and isinstance(password, six.text_type):
            password = password.encode('utf8')

        try:
            membertool.setPassword(password, None, REQUEST=self.request)
        except AttributeError:
            failMessage = _(u'While changing your password an AttributeError '
                            u'occurred. This is usually caused by your user '
                            u'being defined outside the portal.')

            IStatusMessage(self.request).addStatusMessage(
                _(failMessage), type="error"
            )
            return

        # handle PW expiry in Plone's change-password form
        user = api.user.get_current()
        user.setMemberProperties(mapping={
            'reset_password_required': False,
            'password_date': DateTime()
        })

        IStatusMessage(self.request).addStatusMessage(
            _("Password changed"), type="info"
        )



from castle.cms.interfaces import IUploadedToYoutube
from plone.dexterity.browser import edit
from plone.z3cform import layout
from z3c.form.interfaces import HIDDEN_MODE
from castle.cms.services.google import youtube


class VideoEditForm(edit.DefaultEditForm):
    def update(self):
        super(VideoEditForm, self).update()
        if IUploadedToYoutube.providedBy(self.context):
            # can only move these to youtube...
            self.widgets['upload_to_youtube'].mode = HIDDEN_MODE
            self.widgets['youtube_url'].mode = HIDDEN_MODE
        elif youtube.get_oauth_token() is None:
            self.widgets['upload_to_youtube'].mode = HIDDEN_MODE


VideoEditView = layout.wrap_form(VideoEditForm)



from plone import api
from plone.memoize.view import memoize
from castle.cms.browser.syndication import FolderFeed
from plone.app.layout.viewlets.social import SocialTagsViewlet as BaseSocialTagsViewlet
from zope.component.hooks import getSite
from Products.CMFPlone.interfaces.syndication import IFeedItem
from zope.component import queryMultiAdapter
from castle.cms.syndication import DexterityItem


class SocialTagsViewlet(BaseSocialTagsViewlet):
    @memoize
    def _get_tags(self):
        tags = super(SocialTagsViewlet, self)._get_tags()
        site = getSite()
        site_title = api.portal.get_registry_record('plone.site_title', default=None)
        feed = FolderFeed(site)
        item = queryMultiAdapter((self.context, feed), IFeedItem, default=None)
        if item is None:
            item = DexterityItem(self.context, feed)
        finaltags = []
        for tag in tags:
            if site_title and (tag.get('property', '') == 'og:title' or
                                    tag.get('name', '') == 'twitter:title'):
                tag['content'] = u'{} | {}'.format(tag['content'], site_title)
            if item.has_image:
                if tag.get('name', '') == 'twitter:card':
                    # change to large summary
                    tag['content'] = 'summary_large_image'
                if ('image' in tag.get('property', '') or
                        'image' in tag.get('itemprop', '') or
                        'image' in tag.get('name', '')):
                    continue
                finaltags.append(tag)
        if item.has_image:
            finaltags.extend([
                dict(property="og:image", content=item.image_url),
                dict(itemprop="image", content=item.image_url),
                dict(property="og:image:type", content=item.image_type),
                dict(name="twitter:image", content=item.image_url)
            ])
        if item.has_enclosure and item.file_length > 0:
            if item.file_type.startswith('audio'):
                finaltags.extend([
                    dict(name="twitter:card", content="player"),
                    dict(name="twitter:player:width", content="480"),
                    dict(name="twitter:player:height", content="55"),
                    dict(name="twitter:player", content='{}/@@audioplayer'.format(
                        self.get_https_url())),
                    dict(name="twitter:player:stream", content='{}/@@download'.format(
                        self.get_https_url())),
                    dict(name="twitter:player:stream:content_type", content=item.file_type)
                ])
            elif item.file_type.startswith('video'):
                finaltags.extend([
                    dict(name="twitter:card", content="player"),
                    dict(name="twitter:player:width", content="480"),
                    dict(name="twitter:player:height", content="225"),
                    dict(name="twitter:player", content='{}/@@videoplayer'.format(
                        self.get_https_url())),
                    dict(name="twitter:player:stream", content='{}/@@download'.format(
                        self.get_https_url())),
                    dict(name="twitter:player:stream:content_type", content=item.file_type)
                ])

        return finaltags

    def get_https_url(self):
        # Twitter Player Cards require HTTPS resource URLs
        url = self.context.absolute_url()
        return url.replace('http:', 'https:')



from AccessControl.SecurityManagement import newSecurityManager
from castle.cms._scripts import mjson
from castle.cms._scripts.importtypes import get_import_type
from lxml.html import fromstring
from lxml.html import tostring
from OFS.interfaces import IFolder
from plone import api
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.redirector.interfaces import IRedirectionStorage
from plone.app.textfield.value import RichTextValue
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from zope.component import getUtility
from zope.component.hooks import setSite

import argparse
import logging
import os
import transaction
import pdb

logger = logging.getLogger('castle.cms')

# ToDo:
#   - do NOT export images
#       - only images that are referenced in content
#   - lookup reference, if image only used once, add as lead image
#   - other images, if not referenced elsewhere, include inline
#       as data uris
#   - links that go to files should be moved to s3 and be converted
#       into file objects
#   - if default page and only page inside folder, replace folder
#


parser = argparse.ArgumentParser(
    description='...')
parser.add_argument('--site-id', dest='site_id', default='Castle')
parser.add_argument(
    '--export-directory', dest='export_directory', default='export')
parser.add_argument('--import-paths', dest='import_paths', default=False)
parser.add_argument('--skip-paths', dest='skip_paths', default=False)

parser.add_argument('--overwrite', dest='overwrite', default=False)
parser.add_argument('--admin-user', dest='admin_user', default='admin')
parser.add_argument('--ignore-uuids', dest='ignore_uuids', default=False)
parser.add_argument(
    '--stop-if-exception', dest='stop_if_exception', default=False)
parser.add_argument(
    '--pdb-if-exception', dest='pdb_if_exception', default=False)
parser.add_argument('--skip-existing', dest='skip_existing', default=True)
parser.add_argument(
    '--skip-transitioning', dest='skip_transitioning', default=False)
parser.add_argument(
    '--skip-types', dest='skip_types', default=','.join([
        "collective.cover.content",
        "FormFolder",
        "FormMailerAdapter",
        "FormTextField",
        "FormStringField",
        "FormThanksPage",
        "FormSaveDataAdapter",
        "FormSaveData2ContentAdapter",
        "FormSelectionField",
        "Topic"]))
parser.add_argument('--only-types', dest='only_types', default=False)

# Put files/videos/etc in to the repository paths by default,
# Set true to retain exported path
parser.add_argument('--retain-paths', dest='retain_paths', default=False)

args, _ = parser.parse_known_args()

ignore_uuids = args.ignore_uuids
stop_if_exception = args.stop_if_exception
pdb_if_exception = args.pdb_if_exception
retain_paths = args.retain_paths

if args.import_paths:
    import_paths = args.import_paths.split(',')
else:
    import_paths = False
if args.skip_paths:
    skip_paths = args.skip_paths.split(',')
else:
    skip_paths = False


if args.only_types:
    only_types = args.only_types.split(',')
else:
    only_types = False
if args.skip_types:
    skip_types = args.skip_types.split(',')
else:
    skip_types = False

user = app.acl_users.getUser(args.admin_user)  # noqa
try:
    newSecurityManager(None, user.__of__(app.acl_users))  # noqa
except Exception:
    logger.error('Unknown admin user; '
                 'specify an existing Zope admin user with --admin-user '
                 '(default is admin)')
    exit(-1)
site = app[args.site_id]  # noqa
setSite(site)


def traverse(path):
    folder = site  # noqa
    for part in path.strip('/').split('/'):
        if part in folder.objectIds():
            folder = folder[part]
        else:
            return


def relpath(obj):
    return '/'.join(obj.getPhysicalPath())[len(
        '/'.join(site.getPhysicalPath())) + 1:]


_importable_fields = (
    'title',
    'description',
)


def fix_html_images(obj):
    try:
        html = obj.text.raw
    except Exception:
        return
    if not html:
        return
    changes = False
    try:
        dom = fromstring(html)
    except Exception:
        return
    for el in dom.cssselect('img'):
        src = el.attrib.get('src', '')
        if 'resolveuid' not in src:
            continue
        if '@@images' in src:
            continue
        parts = src.split('/')
        uid = parts[1]
        scale = '/'.join(parts[2:])
        if scale:
            scale = scale.replace('image_', '')
        src = 'resolveuid/%s/@@images/image' % uid
        if scale:
            src += '/' + scale
        attribs = {
            'src': src,
            'data-linktype': 'image',
            'data-val': uid
        }
        if scale:
            attribs['data-scale'] = scale
        el.attrib.update(attribs)
        changes = True
    if changes:
        obj.text = RichTextValue(
            tostring(dom), mimeType=obj.text.mimeType,
            outputMimeType=obj.text.outputMimeType)


class CastleImporter(object):
    imported_count = 0

    def do_import(self):
        self.import_folder(args.export_directory, container=site)

    def import_object(self, filepath, container=None):
        fi = open(filepath)
        file_read = fi.read()
        fi.close()
        try:
            data = mjson.loads(file_read)
        except Exception:
            print("Skipping {}; Unable to read JSON data".format(filepath))
            return
        if filepath.endswith('__folder__'):
            filepath = '/'.join(filepath.split('/')[:-1])

        skipped = False
        if data['portal_type'] in skip_types:
            print('Skipping omitted type {type}'
                                            .format(type=data['portal_type']))
            skipped = True
        if only_types and data['portal_type'] not in only_types:
            print("Skipping {type} at {path}, not in only_types."
                                .format(type=data['portal_type'], path=filepath))
            skipped = True
        if import_paths:
            do_import = False
            for import_path in import_paths:
                if filepath.startswith('{}/{}'.format(args.export_directory, import_path)):
                    do_import = True
                if import_path.startswith(filepath[len(args.export_directory):].lstrip('/') + '/'):
                    # Don't skip folders on the way to import_paths
                    do_import = True
            if not do_import:
                print("Skipping {path}, not in import_paths"
                                    .format(path=filepath))
                skipped = True
        if skip_paths:
            for skip_path in skip_paths:
                if filepath.lower().startswith('{}/{}'.format(args.export_directory, skip_path)):
                    print("Skipping {path}, in skip_paths"
                                    .format(path=filepath))
                    skipped = True
        if skipped:
            if os.path.isdir(filepath) and len(os.listdir(filepath)):
                logger.warn('{path} contains additional content that will be '
                            'skipped.'.format(path=filepath))
            return

        original_path = filepath[len(args.export_directory):]
        if retain_paths:
            importtype = get_import_type(data, original_path, 'retain_paths')
        else:
            importtype = get_import_type(data, original_path)
        path = importtype.get_path()

        if container is None:
            logger.warn('Skipped {} because of creation error'.format(filepath))
            return
        _id = path.split('/')[-1]

        create = True
        if _id in container.objectIds():
            if args.overwrite:
                existing = container[_id]
                if IFolder.providedBy(existing):
                    if len(existing.objectIds()):
                        print("OVERWRITE: Deleting non-empty container {path}".format(path=path))
                else:
                    print("OVERWRITE: Deleting content item at {path}".format(path=path))
                api.content.delete(container[_id])
            else:
                create = False

        creation_data = importtype.get_data()
        pc_data = importtype.get_post_creation_data()
        creation_data['container'] = container

        aspect = ISelectableConstrainTypes(container, None)
        if aspect:
            if (aspect.getConstrainTypesMode() != 1 or
                    [creation_data['type']] != aspect.getImmediatelyAddableTypes()):
                aspect.setConstrainTypesMode(1)
                aspect.setImmediatelyAddableTypes([creation_data['type']])
        if create:
            if ignore_uuids and '_plone.uuid' in creation_data:
                del creation_data['_plone.uuid']

            obj = None
            if not args.overwrite and (_id in container.objectIds()):
                print('Skipping {path}, already exists. Use --overwrite to'
                                            ' create anyway.'.format(path=path))
                return
            elif (not ignore_uuids and api.content.get(UID=creation_data['_plone.uuid']) is not None):
                logger.warn('Skipping {path}, content with its UUID already exists.'
                                            'Use --ignore-uuids to create anyway.'
                                                                .format(path=path))
                return
            else:
                try:
                    obj = api.content.create(safe_id=True, **creation_data)
                    print('Created {path}'.format(path=path))
                    self.imported_count += 1
                    if self.imported_count % 50 == 0:
                        print('%i processed, committing' % self.imported_count)
                        transaction.commit()
                except api.exc.InvalidParameterError:
                    if stop_if_exception:
                        logger.error('Error creating content {}'.format(filepath), exc_info=True)
                        if pdb_if_exception:
                            pdb.set_trace()
                        raise
                    logger.error('Error creating content {}'
                                                .format(filepath), exc_info=True)
                    return
    # TODO check default folder pages came over as folder with rich text tile
    # TODO any folder pages without default page should have content listing tile
        else:
            obj = container[_id]
            for key, value in creation_data.items():
                if key not in ('id', 'type'):
                    setattr(obj, key, value)

        if obj is not None:
            if path != original_path:
                storage = getUtility(IRedirectionStorage)
                rpath = os.path.join('/'.join(site.getPhysicalPath()),
                                     original_path.strip('/'))
                storage.add(rpath, "/".join(obj.getPhysicalPath()))

            obj.contentLayout = importtype.layout
            importtype.post_creation(obj, post_creation_data=pc_data)
            if not args.skip_transitioning and data['state']:
                # transition item only if it needs it
                state = api.content.get_state(obj=obj)
                if state != data['state']:
                    try:
                        print('Transitioning %s to %s' % (obj.id, data['state']))
                        api.content.transition(obj, to_state=data['state'])
                    except Exception:
                        logger.error("Error transitioning %s to %s, maybe workflows"
                                     " don't match up" % (obj.id, data['state']))
                        # pass
                        if stop_if_exception:
                            if pdb_if_exception:
                                pdb.set_trace()
                            raise

            # set workflow / review history
            if 'review_history' in data:
                review_history = data['review_history']
                wtool = api.portal.get_tool(name='portal_workflow')
                # loop over all workflow chains (usually only 1)
                for workflow_id in wtool.getChainFor(obj):
                    obj.workflow_history[workflow_id] = review_history
            else:
                logger.warn('No review history on {obj}'.format(obj=obj))

            fix_html_images(obj)
            obj.reindexObject()
            try:
                modification_date = data['data']['modification_date']
                obj.setModificationDate(modification_date)
                obj.reindexObject(idxs=['modified'])
                logger.info('    set modification date to %s' % modification_date)
            except Exception:
                logger.info('Could not set modification date on {obj}'
                                                                .format(obj=obj))
            return obj

    def import_folder(self, path, container):
        this_folder = os.path.join(path, '__folder__')
        if path is not args.export_directory:
            folder = None
            id = path.split('/')[-1:][0]
            if container and id in container.objectIds():
                if args.overwrite:
                    api.content.delete(container[id])
                else:
                    folder = container[id]
            if not folder:
                if os.path.exists(this_folder):
                    folder = self.import_object(this_folder, container)
                else:
                    folder = self.create_plain_folder(id, container)
            container = folder
        folders = []
        objects = []
        for filename in os.listdir(path):
            if filename in ('.DS_Store', '__folder__'):
                continue
            filepath = os.path.join(path, filename)
            if os.path.isdir(filepath):
                folders.append(filepath)
            else:
                objects.append(filepath)
        for path in folders:
            self.import_folder(path, container)
        for path in objects:
            try:
                self.import_object(path, container)
            except Exception:
                logger.error('Error importing {path}'.format(path=filepath),
                                                                exc_info=True)
                if stop_if_exception:
                    if pdb_if_exception:
                        pdb.set_trace()
                    raise

#    app._p_jar.invalidateCache()  # noqa
#    app._p_jar.sync()  # noqa

    def create_plain_folder(self, id, container):
        logger.info("Creating plain Folder (no __folder__ file found), %s" % id)
        folder = api.content.create(
            type='Folder',
            id=id,
            title=id.capitalize(),
            container=container)
        bdata = ILayoutAware(folder)
        bdata.contentLayout = '++contentlayout++castle/folder-query.html'
        if not args.skip_transitioning:
            api.content.transition(folder, to_state='published')
        return folder


if __name__ == '__main__':

    print('------------------------------')
    print('Start importing')
    print('------------------------------')
    if args.overwrite:
        print('------------------------------')
        print('Importing with overwrite enabled')
        print('------------------------------')
    importer = CastleImporter()
    importer.do_import()
    print('Created {count} Content Items'.format(count=importer.imported_count))
    transaction.commit()



from zope.component import getAllUtilitiesRegisteredFor
from castle.cms.fragments.interfaces import IFragmentsDirectory
from castle.cms.fragments import FragmentView
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

import logging


logger = logging.getLogger('castle.cms')


def getFragment(context, request, name):
    utils = getAllUtilitiesRegisteredFor(IFragmentsDirectory)
    utils.sort(key=lambda u: u.order)
    for util in reversed(utils):
        if util.layer is not None:
            if not util.layer.providedBy(request):
                continue
        try:
            return util.get(context, request, name)
        except KeyError:
            pass
    logger.warn('Could not get fragment {0}'.format(name))
    template = ZopePageTemplate('missing',
                                text='<!-- fragment with missing template -->')
    return FragmentView(context, request, name, 'View', template)



import argparse
import json
import os

import transaction
from AccessControl.SecurityManagement import newSecurityManager
from castle.cms.cron.utils import setup_site


try:
    from castle.cms.browser.site.addsite import AddCastleSite
except ImportError:
    # required! This could be getting called from upgrade
    # integration tests
    from castle.cms.browser.addsite import AddCastleSite


app = app  # noqa

parser = argparse.ArgumentParser(
    description='...')
parser.add_argument('--site-id', dest='site_id', default='Castle')
parser.add_argument('--delete', dest='delete', default=False)
args, _ = parser.parse_known_args()

user = app.acl_users.getUser('admin')
newSecurityManager(None, user.__of__(app.acl_users))

if args.delete and args.site_id in app.objectIds():
    print('Deleting site {}'.format(args.site_id))
    app.manage_delObjects([args.site_id])

if args.site_id not in app.objectIds():
    req = app.REQUEST
    req.form = {
        'submitted': 'true',
        'site_id': args.site_id,
        'title': args.site_id
    }
    view = AddCastleSite(app, req)
    view()

    site = app[args.site_id]
    setup_site(site)

    # set registry data
    data = json.loads(os.environ.get('REGISTRY_DATA', '{}'))
    registry = site.portal_registry
    for key, value in data.items():
        registry[key] = value

    transaction.commit()



from castle.cms.theming import renderLayout
from plone.uuid.interfaces import IUUID
from Products.Five import BrowserView
from zope.component.hooks import getSite


class AudioPlayerView(BrowserView):
    tile = 'castle.cms.audiotile'

    @property
    def params(self):
        return 'audio_files:list={uid}'.format(uid=IUUID(self.context))

    def __call__(self):
        self.request.response.setHeader('X-FRAME-OPTIONS', "ALLOWALL")
        return renderLayout(self.context, self.request, self.index(
            portal_url=getSite().absolute_url()
        ))


class VideoPlayerView(AudioPlayerView):
    tile = 'castle.cms.videotile'

    @property
    def params(self):
        return 'video:list={uid}&show_controls=1&autoplay=1'.format(
            uid=IUUID(self.context))



# -*- coding: utf-8 -*-
# for compat with python3, specifically the urllib.parse includes
# noqa because these need to precede other imports
from future.standard_library import install_aliases
install_aliases()  # noqa

from lxml.html import fromstring
import unittest
from urllib.parse import unquote_plus

import boto3
import botocore
from DateTime import DateTime
from castle.cms import archival
from castle.cms.files import aws
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from castle.cms.interfaces import IArchiveManager
from moto import mock_s3
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import setRoles


class TestArchiveManager(unittest.TestCase):
    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))
        man = archival.ArchiveManager()
        self.archive_manager = IArchiveManager(man)

    def test_get_archival_items(self):
        foobar = api.content.create(
            type='Document',
            id='foobar1',
            container=self.portal)
        api.content.transition(foobar, 'publish')
        foobar.setModificationDate(DateTime() - 10)
        foobar.reindexObject(idxs=['modified'])
        api.portal.set_registry_record(
            'castle.archival_number_of_days', 5)
        api.portal.set_registry_record(
            'castle.archival_types_to_archive', ['Document'])

        self.assertEqual(len(self.archive_manager.getContentToArchive()), 1)

    def test_get_archival_items_pays_attention_to_types(self):
        foobar = api.content.create(
            type='Document',
            id='foobar1',
            container=self.portal)
        api.content.transition(foobar, 'publish')
        foobar.setModificationDate(DateTime() - 10)
        foobar.reindexObject(idxs=['modified'])
        api.portal.set_registry_record(
            'castle.archival_number_of_days', 5)
        api.portal.set_registry_record(
            'castle.archival_types_to_archive', ['News Item'])

        self.assertEqual(len(self.archive_manager.getContentToArchive()), 0)


class TestFlashScriptResourceMover(unittest.TestCase):
    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def test_flash_resource_mover_gets_els(self):
        dom = fromstring('''<html>
<head>
<base href="https://www.foobar.com/foobar/"/><!--[if lt IE 7]></base><![endif]-->
</head><body>
<script type="text/javascript">
    AC_FL_RunContent('codebase','http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=9,0,28,0','width','920','height','670','src','foobar','quality','high','wmode','transparent','pluginspage','http://www.adobe.com/shockwave/download/download.cgi?P1_Prod_Version=ShockwaveFlash','movie','foobar' );
    </script></body></html>''')  # noqa
        mover = archival.FlashScriptResourceMover(dom)
        els = mover.get_elements()
        self.assertEqual(len(els), 1)
        self.assertEqual(mover.get_url(els[0]),
                         'https://www.foobar.com/foobar/foobar.swf')
        mover.modify(els[0], "https://www.foobar.com/foobar/foobar.swf")
        self.assertTrue("https://www.foobar.com/foobar/foobar'" in els[0].text)

    def test_flash_resource_mover_gets_els_bad_markup(self):
        dom = fromstring('''<html>
<head>
<base href="https://www.foobar.com/foobar/avatars/avatars"/><!--[if lt IE 7]></base><![endif]-->
</head><body>
<script type="text/javascript">
    // &amp;amp;amp;amp;amp;amp;amp;amp;amp;amp;amp;lt;![CDATA[
    AC_FL_RunContent( 'codebase','http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=9,0,28,0','width','630','height','341','src','avatars_2','quality','high', 'wmode','transparent','pluginspage','http://www.adobe.com/shockwave/download/download.cgi?P1_Prod_Version=ShockwaveFlash','movie','avatars_2?12' ); //end AC code
    // ]]&amp;amp;amp;amp;amp;amp;amp;amp;amp;amp;amp;gt;
    </script></body></html>''')  # noqa
        mover = archival.FlashScriptResourceMover(dom)
        els = mover.get_elements()
        self.assertEqual(len(els), 1)
        self.assertEqual(mover.get_url(els[0]),
                         'https://www.foobar.com/foobar/avatars/avatars_2.swf')
        mover.modify(els[0], "https://www.foobar.com/foobar/avatars/avatars_2.swf")
        self.assertTrue("https://www.foobar.com/foobar/avatars/avatars_2'" in els[0].text)


class TestStorage(unittest.TestCase):
    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.test_access_key = u'AKIAIOSFODNN7EXAMPLE'
        self.test_secret_key = u'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'
        self.test_bucket_name = u'castletest'
        self.test_bucket_endpoint = u'https://s3.amazonaws.com'
        self.test_base_url = u'https://localhost.localdomain/'  # used for test swap
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))
        api.portal.set_registry_record('castle.aws_s3_key', self.test_access_key)
        api.portal.set_registry_record('castle.aws_s3_secret', self.test_secret_key)
        api.portal.set_registry_record('castle.aws_s3_bucket_name', self.test_bucket_name)
        api.portal.set_registry_record('castle.aws_s3_host_endpoint', self.test_bucket_endpoint)
        api.portal.set_registry_record('castle.aws_s3_base_url', self.test_base_url)

    @mock_s3
    def test_move_to_aws(self):
        # this is creating a bucket in the moto/mock s3 service
        s3conn = boto3.resource('s3', endpoint_url=self.test_bucket_endpoint)
        s3conn.create_bucket(Bucket='castletest')
        s3, bucket = aws.get_bucket("castletest")

        storage = archival.Storage(self.portal)
        content = "this is a test"
        content_path = "a/test/path/for/this/test.html"
        content_type = "text/html; charset=utf-8"

        # the key should not be there before we run through this
        self.assertRaises(botocore.exceptions.ClientError, lambda: bucket.Object(content_path).load())

        storage.move_to_aws(content, content_path, content_type)

        try:
            bucket.Object(archival.CONTENT_KEY_PREFIX + content_path).load()
        except botocore.exceptions.ClientError:
            self.fail("object does not exist after move")

    @mock_s3
    def test_move_resource(self):
        # this is creating a bucket in the moto/mock s3 service
        s3conn = boto3.resource('s3', endpoint_url=self.test_bucket_endpoint)
        s3conn.create_bucket(Bucket='castletest')
        s3, bucket = aws.get_bucket("castletest")

        storage = archival.Storage(self.portal)

        testcontent = 'this is some content'
        moveresource = api.content.create(
            type='Document',
            id='moveresource',
            container=self.portal)
        moveresource.content = testcontent
        api.content.transition(moveresource, 'publish')
        vpath = "/moveresource"
        url = self.portal.absolute_url() + vpath

        new_url = storage.move_resource(url, use_vhm=False)

        self.assertIsNotNone(new_url)

        try:
            # trim off, e.g., 'https://s3.amazonaws.com/bucketname'
            # and then convert the path back from the url escaped version
            droppart = "{}/{}/".format(self.test_bucket_endpoint, self.test_bucket_name)
            content_path = unquote_plus(new_url[len(droppart):])
            bucket.Object(content_path).load()
        except botocore.exceptions.ClientError:
            self.fail("object does not exist after move")

        # move by url of content again
        new_url2 = storage.move_resource(url, use_vhm=False)
        self.assertEqual(new_url, new_url2)

        # test for existence of content in aws still
        try:
            # trim off 'https://s3.amazonaws.com/castletest/'
            # and then convert the path back from the url escaped version
            droppart = "{}/{}/".format(self.test_bucket_endpoint, self.test_bucket_name)
            content_path = unquote_plus(new_url[len(droppart):])
            bucket.Object(content_path).load()
        except botocore.exceptions.ClientError:
            self.fail("object does not exist after move")

    def test_archive_replacement_text(self):
        storage = archival.Storage(self.portal)

        storage.replacements = {
            'foobar': 'barfoo'
        }

        self.assertEqual('<html><body><div>barfoo</div></body></html>',
                         storage.apply_replacements('<html><body><div>foobar</div>'
                                                    '</body></html>'))

    def test_archive_replacement_selector(self):
        storage = archival.Storage(self.portal)

        storage.replacements = {
            '.foobar': 'barfoo'
        }

        self.assertEqual('<html><body><div class="foobar">barfoo</div></body></html>',
                         storage.apply_replacements('<html><body><div class="foobar"></div>'
                                                    '</body></html>'))

    def test_archive_transformers(self):
        storage = archival.Storage(self.portal)
        result = storage.transform_content(
            '<html><body><div class="foo">foo</div></body></html>',
            'http://foobar.com')

        self.assertTrue('>bar<' in result)



from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from castle.cms.constants import ALL_SUBSCRIBERS
from castle.cms.constants import ALL_USERS
from html2text import html2text
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from zope.component import getUtility


def get_email_from_address():
    registry = getUtility(IRegistry)
    mail_settings = registry.forInterface(IMailSchema, prefix='plone')
    return mail_settings.email_from_address


def send_email(recipients=None, subject=None, html='', text='', sender=None):
    if isinstance(recipients, basestring):
        recipients = [recipients]

    cleaned_recipients = []
    for recipient in recipients:
        if recipients == ALL_USERS:
            for user in api.user.get_users():
                email = user.getProperty('email')
                if email:
                    cleaned_recipients.append(email)
        elif recipients == ALL_SUBSCRIBERS:
            from castle.cms import subscribe
            cleaned_recipients.extend(subscribe.get_email_addresses())
        else:
            cleaned_recipients.append(recipient)

    if sender is None:
        sender = get_email_from_address()

    if not text and html:
        try:
            text = html2text(html)
        except Exception:
            pass

    for recipient in cleaned_recipients:
        msg = MIMEMultipart('alternative')
        msg['Subject'] = subject
        msg['From'] = sender
        msg['To'] = recipient
        if text:
            part = MIMEText(text, 'plain')
            msg.attach(part)
        if html:
            part = MIMEText(html, 'html')
            msg.attach(part)

        mailhost = api.portal.get_tool('MailHost')
        mailhost.send(msg.as_string())



from Products.CMFCore.utils import getToolByName


PROFILE_ID = 'profile-castle.cms:2_5_15'


def upgrade(context, logger=None):
    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile(PROFILE_ID)



from castle.cms.interfaces.passwordvalidation import ICustomPasswordValidator
from plone import api
from zope.interface import implementer

from castle.cms import _


@implementer(ICustomPasswordValidator)
class EightCharMin(object):
    """ Checks if the password has already been used
    by iterating over the memberdata property password_history_size.
    """

    def __init__(self, context):
        self.context = context

    def validate(self, password, data):
        # Permit empty passwords here to allow registration links.
        # Plone will force the user to set one.
        if password is None:
            return None

        if api.user.is_anonymous():
            # No check for registrations.
            return None

        if len(password) < 8:
            return _(
                u'Minimum 8 characters',
                default=u'Minimum 8 characters'
            )
        return None



from Products.Five import BrowserView
from lxml import etree
from urlparse import urlparse, parse_qs
from plone.app.uuid.utils import uuidToObject


class SlideshowView(BrowserView):

    def get_slides(self):
        slides = []
        root = etree.fromstring(self.context.content)
        tiles = root.findall('.//{http://www.w3.org/1999/xhtml}div[@data-tile]')
        for tile in tiles:
            tile_path = tile.get('data-tile')
            if 'castle.cms.slide/' in tile_path:
                parsed = urlparse(tile_path)
                data = parse_qs(parsed.query)
                slide = {}
                try:
                    slide['text'] = data['text:text'][0]
                except Exception:
                    slide['text'] = None
                try:
                    image = uuidToObject(data['image:list'][0])
                    slide['image'] = image.absolute_url()
                except Exception:
                    slide['image'] = None
                try:
                    video = uuidToObject(data['video:list'][0])
                    slide['video'] = video.absolute_url()
                except Exception:
                    slide['video'] = None
                try:
                    slide['displayType'] = data['display_type'][0]
                except Exception:
                    slide['displayType'] = 'background-image'
                try:
                    slide['title'] = data['title'][0]
                except Exception:
                    slide['title'] = None
                slide['vert'] = data['vert_text_position'][0]
                slide['hor'] = data['hor_text_position'][0]
                slides.append(slide)
        return slides

    def get_id(self):
        return self.context.custom_dom_id or None



# for compat with python3, specifically the urllib.parse includes
# noqa because these need to precede other imports
from future.standard_library import install_aliases
install_aliases()  # noqa

import logging
from datetime import datetime
import StringIO
from time import time
from urllib.parse import urlsplit, urlunsplit, quote, quote_plus

import botocore
import boto3
from collective.celery.utils import getCelery
from persistent.mapping import PersistentMapping
from plone.namedfile.file import NamedBlobFile
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility


logger = logging.getLogger('castle.cms')


CHUNK_SIZE = 12428800
STORAGE_KEY = 'castle.cms.aws'
FILENAME = u'moved-to-s3'
EXPIRES_IN = 30 * 24 * 60 * 60

# define prefix for keys on where to store data in aws
KEY_PREFIX = 'files/'


def get_bucket(s3_bucket=None):
    registry = getUtility(IRegistry)
    s3_id = registry.get('castle.aws_s3_key', None)
    s3_key = registry.get('castle.aws_s3_secret', None)
    if s3_bucket is None:
        s3_bucket = registry.get('castle.aws_s3_bucket_name', None)

    if not s3_id or not s3_key or not s3_bucket:
        return None, None

    s3args = {
        'aws_access_key_id': s3_id,
        'aws_secret_access_key': s3_key,
    }

    endpoint = registry.get('castle.aws_s3_host_endpoint', None)
    if endpoint:
        s3args['endpoint_url'] = endpoint

    s3 = boto3.resource('s3', **s3args)
    bucket = None
    try:
        s3.meta.client.head_bucket(Bucket=s3_bucket)
        bucket = s3.Bucket(s3_bucket)
    except botocore.exceptions.ClientError as e:
        bucket = None
        error_code = e.response['Error']['Code']
        if error_code == '404':
            logger.warning('bucket {name} not found'.format(name=s3_bucket))
        else:
            logger.error('error querying for bucket {name} (code {code})'.format(
                name=s3_bucket,
                code=error_code,
                log_exc=True))

    return s3, bucket


# move a file from castle to aws
def move_file(obj):
    _, bucket = get_bucket()
    if bucket is None:
        return

    # META DATA
    uid = IUUID(obj)
    if not uid:
        logger.info('Could not get uid of object')
        return
    key = KEY_PREFIX + uid
    filename = obj.file.filename
    if not isinstance(filename, unicode):
        filename = unicode(filename, 'utf-8', errors="ignore")
    filename = quote(filename.encode("utf8"))
    disposition = "attachment; filename*=UTF-8''%s" % filename
    size = obj.file.getSize()
    content_type = obj.file.contentType
    extraargs = {
        'ContentType': content_type,
        'ContentDisposition': disposition,
    }

    # Upload to AWS
    # valid modes in ZODB 3, 4 or 5 do not include 'rb' --
    #   see ZODB/blob.py line 54 (or so) for 'valid_modes'
    # note: upload_fileobj() does a multipart upload, which is why
    #   chunked uploading is no longer performed explicitly
    #   see: https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.upload_fileobj  # noqa
    blob_fi = obj.file._blob.open('r')
    bucket.upload_fileobj(blob_fi, key, ExtraArgs=extraargs)

    # Delete data from ZODB, but leave a reference
    if not getCelery().conf.task_always_eager:
        obj._p_jar.sync()
    obj.file = NamedBlobFile(data='', contentType=obj.file.contentType, filename=FILENAME)
    obj.file.original_filename = filename
    obj.file.original_content_type = content_type
    obj.file.original_size = size

    set_permission(obj)


def set_permission(obj):
    s3, bucket = get_bucket()
    if bucket is None:
        return

    uid = IUUID(obj)
    key = KEY_PREFIX + uid

    # check if object is in aws
    s3_obj = bucket.Object(key)
    try:
        s3_obj.load()  # does HEAD request
    except botocore.exceptions.ClientError:
        logger.error(
            'error reading object {key} in bucket {name}'.format(
                key=key,
                name=bucket.name),
            log_exc=True)
        return

    # is the object public?
    perms = obj.rolesOfPermission("View")
    public = False
    for perm in perms:
        if perm["name"] == "Anonymous":
            if perm["selected"] != "":
                public = True
                break

    # set aws public/private and get url
    object_acl = s3_obj.Acl()
    if public:
        object_acl.put(ACL='public-read')
        # a public URL is not fetchable with no expiration, apparently
        expires_in = 0
        scheme, netloc, path, qs, anchor = urlsplit(s3.meta.client.meta.endpoint_url)
        # this makes sure there are no empty segments, and then appends
        # the bucket name and quoted key value (which is fine, and usable
        # with aws api)
        newpath = "/".join(filter(None, path.split("/")) + [bucket.name, quote_plus(key)])
        url = urlunsplit((scheme, netloc, newpath, qs, anchor))
    else:
        object_acl.put(ACL='private')
        expires_in = EXPIRES_IN
        params = {
            'Bucket': bucket.name,
            'Key': key,
        }
        url = s3_obj.meta.client.generate_presigned_url(
            'get_object',
            Params=params,
            ExpiresIn=expires_in)

    annotations = IAnnotations(obj)
    info = annotations.get(STORAGE_KEY, PersistentMapping())
    info.update({
        'url': url,
        'expires_in': expires_in,
        'generated_on': time()
    })
    annotations[STORAGE_KEY] = info


# TODO: determine if the file obj in plone should also be deleted
# at this time, or document why it isn't, maybe. Content is all
# moved to aws anyway.
def delete_file(uid):
    _, bucket = get_bucket()
    if bucket is None:
        return

    key_name = KEY_PREFIX + uid
    try:
        key = bucket.Object(key_name)
        key.delete()
    except botocore.exceptions.ClientError:
        logger.error(
            'error deleting object {key} in bucket {name}'.format(
                key=key,
                name=bucket.name),
            log_exc=True)


def uploaded(obj):
    try:
        annotations = IAnnotations(obj)
    except TypeError:
        return False
    info = annotations.get(STORAGE_KEY, PersistentMapping())
    return 'url' in info


def swap_url(url, registry=None, base_url=None):
    """
    aws url: https://s3-us-gov-west-1.amazonaws.com/bucketname/archives/path/to/resource
    base url: http://foo.com/
    swapped: http://foo.com/archives/path/to/resource
    """
    if base_url is None:
        if registry is None:
            registry = getUtility(IRegistry)
        base_url = registry.get('castle.aws_s3_base_url', None)

    if base_url:
        basescheme, basenetloc, basepath, baseqs, baseanchor = urlsplit(base_url.decode('utf-8'))
        scheme, netloc, path, qs, anchor = urlsplit(url.decode('utf-8'))
        # make sure there are no empty segements
        # and also drop the first segment -- should be the bucket, which we
        # don't need to pass on to the final url
        newpath = u"/".join(filter(None, path.split(u'/'))[1:])
        url = urlunsplit((basescheme, basenetloc, newpath, qs, anchor))

    return url


def _one(val):
    """Wonky, values returning tuple sometimes?"""
    if type(val) in (tuple, list, set):
        if len(val) > 0:
            val = val[0]
        else:
            return
    return val


def get_url(obj):
    annotations = IAnnotations(obj)
    info = annotations.get(STORAGE_KEY, PersistentMapping())

    if info['expires_in'] != 0:
        # not public, check validity of url
        generated_on = info['generated_on']
        expires = datetime.fromtimestamp(generated_on + info['expires_in'])
        if datetime.utcnow() > expires:
            # need a new url
            s3, bucket = get_bucket()
            if bucket is not None:
                uid = IUUID(obj)
                key = KEY_PREFIX + uid
                params = {
                    'Bucket': bucket.name,
                    'Key': key,
                }
                url = s3.meta.client.generate_presigned_url(
                    'get_object',
                    Params=params,
                    ExpiresIn=EXPIRES_IN)

                info.update({
                    'url': url,
                    'generated_on': time(),
                    'expires_in': EXPIRES_IN
                })
                annotations[STORAGE_KEY] = info

    url = _one(info['url'])
    if not url.startswith('http'):
        url = 'https:' + url

    # need to make sure the url is formatted correctly
    # the main one we've seen is extra slashes in the path
    scheme, netloc, path, qs, anchor = urlsplit(url)
    # this will get rid of any empty segments
    path = "/".join(filter(None, path.split('/')))
    finalurl = urlunsplit((scheme, netloc, path, qs, anchor))
    return swap_url(finalurl)


def create_or_update(bucket, key, content_type, data, content_disposition=None, make_public=True):
    extraargs = {
        'ContentType': content_type,
    }
    if content_disposition is not None:
        extraargs['ContentDisposition'] = content_disposition
    contentio = StringIO.StringIO(data)
    bucket.upload_fileobj(contentio, key, ExtraArgs=extraargs)
    if make_public:
        s3_obj = bucket.Object(key)
        object_acl = s3_obj.Acl()
        object_acl.put(ACL='public-read')


def create_if_not_exists(bucket, key, content_type, data, content_disposition=None, make_public=True):
    try:
        # does a head request for a single key, will throw an error
        # if not found (or elsewise)
        bucket.Object(key).load()
    except botocore.exceptions.ClientError:
        # the object hasn't been pushed to s3 yet, so do that
        # create/update
        create_or_update(
            bucket,
            key,
            content_type,
            data,
            content_disposition=content_disposition,
            make_public=make_public)



import logging
import math
import os
import Queue
import threading
import time
import traceback
import urlparse
from datetime import datetime
from datetime import timedelta

import requests
import requests.exceptions
from requests.packages.urllib3.exceptions import InsecureRequestWarning

import sqlalchemy.exc
from lxml import html
from pylru import lrudecorator
from sqlalchemy import Boolean
from sqlalchemy import Column
from sqlalchemy import DateTime
from sqlalchemy import ForeignKey
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy.orm import sessionmaker


requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

engine = create_engine(os.environ.get('LINK_REPORT_DB', 'sqlite://'))
Base = declarative_base()

logger = logging.getLogger('castle.cms')
USER_AGENT = 'Castle CMS/2.0'


requests.utils.default_headers().update({
    'User-Agent': USER_AGENT
})

_ignored = [
    'javascript:',
    'mailto:',
    'data:',
    'https://twitter.com/intent/tweet',
    'https://www.facebook.com/sharer/sharer.php',
    'http://www.addthis.com',
    'http://www.captcha.net',
    'https://supervisord.org',
    'http://pinterest.com',
    'http://www.facebook.com/sharer',
    'http://reddit.com/submit',
    'http://digg.com/submit',
    'http://www.stumbleupon.com/submit',
    'http://www.linkedin.com/shareArticle',
    'https://plus.google.com/share',
    'http://twitter.com/intent'
]
_ignored_views = ['@@search']
_local = threading.local()


class Url(Base):
    __tablename__ = 'urls'

    url = Column(String, primary_key=True, index=True)
    last_checked_date = Column(DateTime, index=True)
    added_date = Column(DateTime)
    status_code = Column(Integer, index=True)
    content_type = Column(String, nullable=True)
    content_length = Column(String, nullable=True)
    parse_error = Column(String, nullable=True)
    ssl_verified = Column(Boolean, default=True)
    final_url = Column(String, nullable=True, index=True)


class Link(Base):
    __tablename__ = 'links'

    site_id = Column(String, primary_key=True, index=True)
    url_to = Column(String, ForeignKey(Url.url), primary_key=True, index=True)
    url_from = Column(String, ForeignKey(Url.url), primary_key=True, index=True)
    last_found = Column(DateTime)

    _url_to = relationship("Url", foreign_keys=[url_to])
    _url_from = relationship("Url", foreign_keys=[url_from])


def init():
    Base.metadata.create_all(engine)


def get_session():
    if not hasattr(_local, 'session'):
        Session = sessionmaker(bind=engine)
        _local.session = Session()
    return _local.session


_worker_queue = Queue.Queue()
_consumer_queue = Queue.Queue()
_exiting = threading.Event()


def parse_url_worker():
    while True:
        try:
            url = _worker_queue.get(timeout=0.5)
        except Queue.Empty:
            if _exiting.is_set():
                return
            continue

        _worker_queue.task_done()
        try:
            resp = requests.get(
                url, allow_redirects=True, timeout=10,
                stream=True, verify=False, headers={
                    'User-Agent': USER_AGENT
                })
        except Exception as ex:
            resp = ex
        _consumer_queue.put((url, resp))


_exception_status_codes = {
    requests.exceptions.ConnectionError: -2,
    requests.exceptions.ConnectTimeout: -599,
    requests.exceptions.TooManyRedirects: -301
}

status_codes_info = {
    -2: 'Unknown connection error',
    -200: 'nodename nor servname provided, or not known',
    -599: 'Connection timeout',
    -301: 'Too many redirects'
}


class Reporter(object):

    batch_size = 20
    running = True

    def __init__(self, site, recheck_delay=60 * 60 * 24 * 7):
        self.site = site
        self.recheck_delay = recheck_delay
        self.session = get_session()
        self.base = site.portal_registry['plone.public_url']
        if self.base:
            self.base = self.base.rstrip('/')
        self.done = 0
        self.error = 0
        self.queued = 0
        self.found = 0
        self.threads = []
        self.cache = {}

    def join(self):
        _exiting.set()
        self.running = False
        for thread in self.threads:
            thread.join()

    def get_next(self, limit=1):
        now = datetime.utcnow()
        relative = now - timedelta(seconds=self.recheck_delay)
        result = self.session.query(Url, Link.url_to).join(
            Link, Link.url_to == Url.url).filter(
                Url.last_checked_date < relative,
                Link.site_id == self.site.getId()
            ).order_by(Url.last_checked_date).limit(limit)
        return [r[0] for r in result]

    @property
    def valid(self):
        if not self.base:
            return False
        return True

    def __call__(self):
        if not self.valid:
            return

        for _ in range(int(math.ceil(self.batch_size / 3))):
            thread = threading.Thread(target=parse_url_worker)
            thread.start()
            self.threads.append(thread)

        try:
            # just make sure db is setup
            self.get_next()
        except sqlalchemy.exc.OperationalError as ex:
            if 'no such table' in ex.message:
                init()
            else:
                raise

        self.add_link(self.base, self.base)

        while self.running:
            if self.consume() == 0:
                break

        # attempt to retry errors... just once
        self.session.query(Url).filter(
            Url.status_code.in_([-1, -2, 400, 403, 401, 429])).update(
            {'last_checked_date': datetime(1984, 1, 1)},
            synchronize_session=False)
        self.session.expunge_all()

        while self.running:
            if self.consume() == 0:
                break

        self.join()
        while _consumer_queue.qsize() > 0:
            url, resp = _consumer_queue.get()
            if url in self.cache:
                url = self.cache.pop(url)
                self.check_url(url, resp)

    def consume(self):
        urls = self.get_next(self.batch_size)

        for url in urls:
            self.cache[url.url] = url
            _worker_queue.put(url.url)

        while _worker_queue.qsize() > int(math.ceil(self.batch_size / 4)):
            # let it work while we keep moving
            time.sleep(0.1)

        while _consumer_queue.qsize() > 0:
            url, resp = _consumer_queue.get()
            if url in self.cache:
                url = self.cache.pop(url)
                self.check_url(url, resp)
            _consumer_queue.task_done()
        return len(urls)

    def handle_error(self, request, exception):
        return exception

    def check_url(self, url, resp):
        self.done += 1
        if self.done % 500 == 0:
            # clear cache every so often
            self.session.expire_all()

        if isinstance(resp, Exception):
            self.error += 1
            logger.error(
                'Error processing url: {}\n{}'.format(url.url, resp))
            self.update_url(url, None, exception=resp)
        else:
            logger.info('({}/{}/{}/{}): Processing ({}) {}'.format(
                self.found, self.queued, self.done, self.error,
                resp.status_code, url.url))
            if resp.status_code == 429:
                # slow it down since we're getting limited...
                time.sleep(0.2)
            try:
                override_status_code = None
                if '/acl_users/' in resp.url or '?came_from=' in resp.url:
                    override_status_code = 403
                elif resp.status_code == 200:
                    if (resp.url.startswith(self.base) and
                            'html' in resp.headers.get('Content-Type', '')):
                        self.parse_response(resp, url.url)

                self.update_url(url, resp,
                                override_status_code=override_status_code)
            except Exception:
                logger.error('Could not update url {}'.format(url.url), exc_info=True)
            finally:
                resp.close()

    def update_url(self, url, resp=None, exception=None, override_status_code=None):
        url.last_checked_date = datetime.utcnow()
        final_url = None
        if resp is not None:
            url.status_code = override_status_code or resp.status_code
            url.content_type = resp.headers.get('Content-Type')
            url.content_length = resp.headers.get('Content-Length')
            url.parse_error = None
            if url.url != resp.url:
                final_url = resp.url
        else:
            status_code = _exception_status_codes.get(type(exception), -1)
            if status_code == -2:
                if status_codes_info[-200] in str(exception):
                    status_code = -200
            url.status_code = status_code
            url.content_type = None
            url.content_length = None
            url.parse_error = traceback.format_exc(exception)
        url.final_url = final_url
        try:
            self.session.add(url)
            self.session.commit()
        except Exception:
            self.session.rollback()
            raise

    def parse_response(self, resp, from_url):
        base = resp.url
        dom = html.fromstring(resp.content)
        base_tag = dom.cssselect('base')
        if len(base_tag) > 0:
            base_tag = base_tag[0]
            base = base_tag.attrib('href')
            if not base.startswith('http://') and not base.startswith('https://'):
                base = urlparse.urljoin(resp.url, base)

        for anchor in dom.cssselect('a,img'):
            if 'href' not in anchor.attrib and 'src' not in anchor:
                continue
            url = anchor.attrib.get('href', anchor.attrib.get('src'))
            url = url.split('#')[0]
            if not url:
                continue
            if not url.startswith('http://') and not url.startswith('https://'):
                # make absolute url
                url = urlparse.urljoin(base, url)

            drop = False
            for ignore in _ignored:
                if url.startswith(ignore):
                    drop = True
                    break
            for view in _ignored_views:
                if url.endswith(view):
                    drop = True
                    break
            if drop:
                continue

            if not url.startswith('http://') and not url.startswith('https://'):
                logger.warning('Invalid url: {}'.format(url))
                continue

            self.add_link(url, from_url)

    @lrudecorator(5000)
    def add_url(self, url):
        error = False
        try:
            self.session.identity_map[(Url, (url,))]
            # no key error, we already have object, ignore
        except KeyError:
            url_ob = Url(
                url=url,
                last_checked_date=datetime(1984, 1, 1),
                added_date=datetime.utcnow(),
                status_code=-1
            )
            try:
                self.session.add(url_ob)
                self.session.commit()
                self.queued += 1
            except sqlalchemy.exc.IntegrityError:
                # we're okay if we already have a link object in db!
                error = True
            except Exception:
                error = True
                logger.error('Unhandled error updating url object', exc_info=True)
                raise
            finally:
                if error:
                    self.session.rollback()

    @lrudecorator(5000)
    def add_link(self, url, from_url):
        if '/acl_users/' in url or '?came_from=' in url:
            # ignore parsing these guys
            return

        if url[0] == '/':
            # make full url
            url = urlparse.urljoin(from_url, url)

        url = url.rstrip('/')
        self.add_url(url)

        error = False
        try:
            link_ob = Link(
                site_id=self.site.getId(),
                url_to=url,
                url_from=from_url,
                last_found=datetime.utcnow()
            )
            self.session.add(link_ob)
            self.session.commit()
            self.found += 1
        except sqlalchemy.exc.IntegrityError:
            self.session.rollback()
            link_ob = self.session.query(Link).filter(
                Link.site_id == self.site.getId(),
                Link.url_to == url,
                Link.url_from == from_url
            ).first()
            link_ob.last_found = datetime.utcnow()
            self.session.add(link_ob)
            try:
                self.session.commit()
            except Exception:
                logger.error('Error commit link object', exc_info=True)
                error = True
        except Exception:
            error = True
            logger.error('Unhandled error updating link object', exc_info=True)
            raise
        finally:
            if error:
                self.session.rollback()



import logging

from plone import api
from plone.app.querystring import queryparser
from plone.app.querystring.interfaces import IParsedQueryIndexModifier
from zope.component import getUtilitiesFor


logger = logging.getLogger('castle.cms')


def parse_query_from_data(data, context=None):
    if context is None:
        context = api.portal.get()
    query = data.get('query', {}) or {}
    try:
        parsed = queryparser.parseFormquery(context, query)
    except KeyError:
        logger.info('Error parsing query {}'.format(repr(query)))
        parsed = {}

    index_modifiers = getUtilitiesFor(IParsedQueryIndexModifier)
    for name, modifier in index_modifiers:
        if name in parsed:
            new_name, query = modifier(parsed[name])
            parsed[name] = query
            # if a new index name has been returned, we need to replace
            # the native ones
            if name != new_name:
                del parsed[name]
                parsed[new_name] = query

    if data.get('sort_on'):
        parsed['sort_on'] = data['sort_on']
    if data.get('sort_reversed', False):
        parsed['sort_order'] = 'reverse'
    return parsed



from castle.cms.tiles.base import BaseTile
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.interface import Interface


class GoogleTagManagerTile(BaseTile):

    def render(self):
        registry = getUtility(IRegistry)
        enabled = registry.get('castle.gtm_enabled', None)
        if enabled:
            return self.index()
        return ''

    @property
    def gtm_id(self):
        registry = getUtility(IRegistry)
        gtm_id = registry.get('castle.gtm_id', None)
        return gtm_id


class GoogleTagManagerTileSchema(Interface):
    pass



from castle.cms.behaviors.location import ILocation
from imghdr import what
from mimetypes import guess_type
from OFS.Image import Image
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.contenttypes.behaviors.richtext import IRichText
from plone.app.dexterity.behaviors.metadata import (IBasic, ICategorization,
                                                    IDublinCore, IPublication)
from plone.app.event.dx.behaviors import (IEventAttendees, IEventBasic,
                                          IEventContact, IEventLocation)
from plone.app.textfield.value import RichTextValue
from plone.event.utils import pydt
from plone.namedfile.file import NamedBlobFile, NamedBlobImage
from StringIO import StringIO

from zope.interface import Interface
from zope.component import getUtilitiesFor, getGlobalSiteManager

import logging
import base64
import OFS
import re
import pdb

logger = logging.getLogger('castle.cms')

site_manager = getGlobalSiteManager()


class IImportType(Interface):
    pass


FOLDER_DEFAULT_PAGE_LAYOUT = u"""
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
                      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en"
      data-layout="./@@page-site-layout">
  <body>
    <div data-panel="content">
      <div class="mosaic-grid-row"
           data-grid='{"type": "row"}'>
        <div class="mosaic-grid-cell mosaic-width-full mosaic-position-leftmost"
             data-grid='{"type": "cell", "info":{"xs": "true", "sm": "true", "lg": "true",
                         "pos": {"x": 1, "width": 12}}}'>
          <div class="movable removable mosaic-tile mosaic-IDublinCore-title-tile">
          <div class="mosaic-tile-content">
          <div data-tile="./@@plone.app.standardtiles.field?field=IDublinCore-title"></div>
          </div>
          </div>
          <div class="movable removable mosaic-tile mosaic-IDublinCore-description-tile">
          <div class="mosaic-tile-content">
          <div data-tile="./@@plone.app.standardtiles.field?field=IDublinCore-description"></div>
          </div>
          </div>
          <div class="movable removable mosaic-tile mosaic-text-tile">
          <div class="mosaic-tile-content">
            %s
          </div>
          </div>
        </div>
      </div>
    </div>
  </body>
</html>
"""

dublin = 'plone.app.dexterity.behaviors.metadata.IDublinCore'
basic = 'plone.app.dexterity.behaviors.metadata.IBasic'
categorization = 'plone.app.dexterity.behaviors.metadata.ICategorization'
layoutaware = 'plone.app.blocks.layoutbehavior.ILayoutAware'
publication = 'plone.app.dexterity.behaviors.metadata.IPublication'

_types = {}


def register_import_type(name, klass):
    site_manager.registerUtility(klass, IImportType, name)


def is_base64(s):
    if hasattr(s, 'len'):
        return (len(s) % 4 == 0) and re.match('^[A-Za-z0-9+/]+[=]{0,2}$', s)
    elif hasattr(s, 'size'):
        return (s.size % 4 == 0) and re.match('^[A-Za-z0-9+/]+[=]{0,2}$', s)
    else:
        return (len(s) % 4 == 0) and re.match('^[A-Za-z0-9+/]+[=]{0,2}$', s)


def decode_file_data(data):
    if isinstance(data, OFS.Image.Image):
        data = data.data
        if isinstance(data, OFS.Image.Pdata):
            return str(data)
        return data
    if isinstance(data, StringIO):
        data = data.buf
    if is_base64(data):
        return base64.b64decode(data)
    return data


def to_unicode(s):
    if not isinstance(s, unicode):
        s = s.decode('utf8')
    return s


def _generate_file_repo_object_id(path):
    path = path.lower()
    if path[0] == '/':
        path = path[1:]
    for _type in ('file', 'audio', 'video', 'videos', 'image', 'images'):
        path = path.replace(
            '%s-repository/' % _type, '').replace('/' + _type, '')
    return path.replace('/', '-')


def DateTime_to_datetime(val):
    if hasattr(val, 'asdatetime'):
        return pydt(val)
    return val


class BaseImportType(object):
    layout = ''
    fields_mapping = {}
    data_converters = {}
    lead_image_field_names = (
        'image', 'leadImage',
        'plone.app.contenttypes.behaviors.leadimage.ILeadImage')
    lead_image_caption_field_names = ('leadCaption', 'leadImage_caption')
    behavior_data_mappers = ()

    def __init__(self, data, path, *args):
        self.data = data
        self.field_data = data['data']
        self.path = path
        self.original_type = data['portal_type']
        if data.get('is_default_page'):
            # change to be folder instead of sub-item...
            self.path = '/'.join(path.split('/')[:-1])
            self.data['portal_type'] = 'Folder'
            # disabled for now, can think about how to use this more properly...
            # if data.get('has_sibling_pages'):
            #     self.data['portal_type'] = 'Folder'

    def get_path(self):
        return self.path

    def get_data(self):
        data = {
            '_plone.uuid': self.data['uid']
        }
        try:
            id = self.field_data['id']
        except Exception:
            try:
                id = self.field_data['plone.app.dexterity.behaviors.id.IShortName']['id']
            except Exception:
                id = self.path.split('/')[-1]
        try:
            title = self.field_data['title']
        except Exception:
            try:
                title = self.field_data['plone.app.content.interfaces.INameFromTitle']['title']
            except Exception:
                try:
                    title = self.field_data[dublin]['title']
                except Exception:
                    pass

        try:
            description = self.field_data['description']
        except Exception:
            try:
                description = self.field_data[dublin]['description']
            except Exception:
                try:
                    description = self.field_data[basic]['description']
                except Exception:
                    pdb.set_trace()
        return dict(
            id=id,
            type=self.data['portal_type'],
            title=title,
            description=description,
            **data)

    def get_post_creation_data(self):
        pass

    def post_creation(self, obj, pdb_if_exception=False, post_creation_data=None):
        if obj is None:
            return
        field_data = self.field_data
        bdata = ILayoutAware(obj, None)
        if bdata:
            try:
                bdata.contentLayout = self.layout
            except Exception:
                bdata.contentLayout = '++contentlayout++default/document.html'
        bdata = IRichText(obj, None)
        if bdata:
            try:
                bdata.text = RichTextValue(field_data['text'], 'text/html', 'text/html')
            except Exception:
                try:
                    bdata.text = RichTextValue(
                        field_data['plone.app.contenttypes.behaviors.richtext.IRichText']['text'], # noqa
                        'text/html', 'text/html').raw
                except Exception:
                    bdata.text = ''

        bdata = IBasic(obj, None)
        if bdata:
            try:
                bdata.title = field_data['title']
            except Exception:
                try:
                    bdata.title = field_data['plone.app.content.interfaces.INameFromTitle']['title']
                except Exception:
                    bdata.description = field_data[dublin]['title']
            try:
                bdata.description = field_data['description']
            except Exception:
                try:
                    bdata.description = field_data[dublin]['description']
                except Exception:
                    bdata.description = field_data[basic]['description']
        else:
            try:
                obj.title = field_data['title']
                obj.description = field_data['description']
            except Exception:
                obj.title = field_data[dublin]['title']
                obj.description = field_data[dublin]['description']

        bdata = ICategorization(obj, None)
        if bdata:
            try:
                bdata.subjects = field_data['subject']
            except Exception:
                try:
                    bdata.subjects = self.field_data[dublin]['subjects']
                except Exception:
                    try:
                        bdata.subjects = self.field_data[categorization]['subjects']
                    except Exception:
                        pass  # no keywords found

            bdata = IPublication(obj)
            try:
                if field_data['effectiveDate']:
                    bdata.effective = pydt(field_data['effectiveDate'])
            except Exception:
                try:
                    if field_data[dublin]['effective']:
                        bdata.effective = pydt(field_data[dublin]['effective'])
                except Exception:
                    try:
                        if field_data[publication]['effective']:
                            bdata.effective = pydt(field_data[publication]['effective'])
                    except Exception:
                        bdata.effective = None

        ldata = ILocation(obj, None)
        if ldata:
            if field_data.get('location'):
                ldata.locations = [field_data['location']]

            if field_data.get('newsLocation'):
                if ldata.locations:
                    ldata.locations.append(field_data['newsLocation'])
                else:
                    ldata.locations = [field_data['newsLocation']]

        try:
            obj.modification_date = field_data['modification_date']
        except Exception:
            try:
                obj.modification_date = obj.modified()
            except Exception:
                obj.modification_date = None
        try:
            obj.creation_date = field_data['creation_date']
        except Exception:
            try:
                obj.creation_date = obj.created()
            except Exception:
                obj.creation_date = None

        bdata = IDublinCore(obj, None)
        if bdata:
            if IDublinCore.__identifier__ in field_data:
                dublin_core = field_data[IDublinCore.__identifier__]
                bdata.expires = dublin_core['expires']
                bdata.rights = dublin_core['rights']
                bdata.creators = tuple(dublin_core['creators'])
                bdata.language = dublin_core['language']
                bdata.effective = pydt(dublin_core['effective'])
                bdata.subjects = dublin_core['subjects']
                bdata.contributors = tuple(dublin_core['contributors'])
            else:
                bdata.expires = pydt(field_data.get('expirationDate'))
                bdata.rights = field_data.get('rights')
                creators = field_data.get('creators')
                bdata.creators = tuple(creators) if creators else ()
                language = field_data.get('language')
                bdata.language = language if language is not None else ""
                bdata.effective = pydt(field_data.get('effectiveDate'))
                bdata.subjects = field_data.get('subject')
                contributors = field_data.get('contributors')
                bdata.contributors = tuple(contributors) if contributors else ()

        bdata = ILayoutAware(obj, None)
        if bdata:
            if self.data['portal_type'] == 'Folder' and (self.field_data.get('text') or '').strip():
                bdata.content = FOLDER_DEFAULT_PAGE_LAYOUT % self.field_data['text']
                # need to explicitly reset contentLayout value because this data
                # could be overwritten
                bdata.contentLayout = None
            elif self.layout:
                if layoutaware in field_data and 'contentLayout' in field_data[layoutaware]:
                    bdata.contentLayout = field_data[layoutaware]['contentLayout']
                if layoutaware in field_data and 'content' in field_data[layoutaware]:
                    bdata.content = field_data['plone.app.blocks.layoutbehavior.ILayoutAware']['content']
                if 'rendered_layout' in self.data['data']:
                    bdata.rendered_layout = self.data['data']['rendered_layout']

        inv_field_mapping = {v: k for k, v in self.fields_mapping.iteritems()}
        for IBehavior, field_name in self.behavior_data_mappers:

            original_field_name = inv_field_mapping.get(field_name, field_name)

            if original_field_name not in self.field_data:
                # data not here...
                continue

            behavior = IBehavior(obj, None)
            if behavior is None:
                # behavior not valid for obj type
                continue

            val = self.field_data[original_field_name]

            if field_name in self.data_converters:
                val = self.data_converters[field_name](val)

            setattr(behavior, field_name, val)

        # handle lead images
        for field_name in self.lead_image_field_names:
            if self.field_data.get(field_name):
                if field_name == 'plone.app.contenttypes.behaviors.leadimage.ILeadImage':
                    im_obj = self.field_data.get(field_name)['image']
                else:
                    im_obj = self.field_data.get(field_name)
                if hasattr(im_obj, 'read'):
                    im_data = im_obj.read()
                else:
                    im_data = im_obj

                if not im_data:
                    continue

                filename = self.field_data.get('image_filename')
                if not filename:
                    if hasattr(im_obj, 'filename'):
                        filename = im_obj.filename
                    else:
                        filename = self.field_data['id']
                obj.image = im_data

                is_stringio = isinstance(im_obj, StringIO)
                if is_stringio:
                    namedblobimage_data = im_data
                elif isinstance(im_obj, Image):
                    namedblobimage_data = im_data.data
                else:
                    if pdb_if_exception:
                        pdb.set_trace()
                    logger.info("    lead image is type %s" % type(im_obj))
                obj.image = NamedBlobImage(data=namedblobimage_data, contentType='', filename=filename)

                if hasattr(obj.image, 'contentType') and isinstance(obj.image.contentType, unicode):
                    obj.image.contentType = obj.image.contentType.encode('ascii')
                else:
                    if isinstance(im_obj, Image):
                        data = im_obj.data
                    else:
                        data = im_obj.buf
                    image_type = what('', h=data)
                    if image_type in ['png', 'bmp', 'jpeg', 'xbm', 'tiff', 'gif']:
                        obj.image.contentType = 'image/%s' % image_type
                    elif image_type == 'rast':
                        obj.image.contentType = 'image/cmu-raster'
                    elif image_type == 'ppm':
                        obj.image.contentType = 'image/x-portable-pixmap'
                    elif image_type == 'pgm':
                        obj.image.contentType = 'image/x-portable-greymap'
                    elif image_type == 'pbm':
                        obj.image.contentType = 'image/x-portable-bitmap'
                    elif image_type == 'rgb':
                        obj.image.contentType = 'image/x-rgb'
                    else:
                        # look at filename extension
                        contentType, encoding = guess_type(obj.image.filename, strict=False)
                        if contentType:
                            obj.image.contentType = contentType
                        else:
                            logger.info("Unknown image type {};"
                                        " defaulting to jpeg"
                                        .format(image_type))
                            pdb.set_trace()
                            obj.image.contentType = 'image/jpeg'  # default
                for caption_field_name in self.lead_image_caption_field_names:
                    if caption_field_name in self.field_data:
                        obj.imageCaption = self.field_data.get(caption_field_name)


class DocumentType(BaseImportType):
    layout = '++contentlayout++default/document.html'


register_import_type('Document', DocumentType)


class FolderType(BaseImportType):
    layout = '++contentlayout++castle/folder.html'

    def __init__(self, data, path, *args):
        super(FolderType, self).__init__(data, path, *args)
        self.data['portal_type'] = 'Folder'


register_import_type('Folder', FolderType)
register_import_type('PressRoom', FolderType)


class NewsItemType(BaseImportType):
    layout = '++contentlayout++castle/news_item.html'

    def __init__(self, data, path, *args):
        super(NewsItemType, self).__init__(data, path, *args)
        self.data['portal_type'] = 'News Item'


register_import_type('News Item', NewsItemType)
register_import_type('PressClip', NewsItemType)
register_import_type('PressRelease', NewsItemType)


class FileType(BaseImportType):
    layout = None

    def __init__(self, data, path, *args):
        super(FileType, self).__init__(data, path, *args)
        filename = self.field_data.get('file_filename')
        if filename is None:
            filename = path.split('/')[-1]
        if filename is None:
            filename = self.field_data['title']
        try:
            _, ext = filename.lower().rsplit('.', 1)
        except Exception:
            ext = 'pdf'  # provide something so it ends up in file repository

        if ext in ('wav', 'mp3'):
            self.data['portal_type'] = 'Audio'
            repo_path = '/audio-repository/'
        elif ext in ['webm', 'ogv', 'avi', 'wmv', 'm4v',
                     'mpg', 'mpeg', 'flv', 'mp4', 'mov']:
            self.data['portal_type'] = 'Video'
            repo_path = '/video-repository/'
        else:
            repo_path = '/file-repository/'
            if ext == 'pdf':
                self.layout = 'documentviewer'

        if 'retain_paths' in args:
            self.path = path
        else:
            _id = _generate_file_repo_object_id(path)
            self.field_data['id'] = _id
            self.path = repo_path + _id

    def get_data(self):
        data = super(FileType, self).get_data()
        filename = self.field_data.get('file_filename')
        if filename is None or not filename:
            try:
                filename = self.field_data['file'].filename
            except Exception:
                pass
        if filename is None or not filename:
            filename = self.field_data['id']
        if 'file' in self.field_data:
            data['file'] = NamedBlobFile(
                data=decode_file_data(self.field_data['file']),
                filename=to_unicode(filename))
        return data


register_import_type('File', FileType)


class ImageType(BaseImportType):

    def __init__(self, data, path, *args):
        super(ImageType, self).__init__(data, path, *args)
        _id = _generate_file_repo_object_id(path)
        self.field_data['id'] = _id
        self.path = '/image-repository/' + _id


register_import_type('Image', ImageType)


class VideoType(FileType):

    def __init__(self, data, path, *args):
        super(FileType, self).__init__(data, path, *args)
        if self.original_type == 'WildcardVideo':
                self.data['portal_type'] = 'Video'
                repo_path = '/video-repository/'

        if 'retain_paths' in args:
            self.path = path
        else:
            _id = _generate_file_repo_object_id(path)
            self.field_data['id'] = _id
            self.path = repo_path + _id

    def get_data(self):
        data = super(FileType, self).get_data()
        if self.original_type == 'WildcardVideo':
            video_data = self.data['data']['wildcard.media.behavior.IVideo']
            if video_data['video_file']:
                video_blob = video_data['video_file']
            elif video_data['video_file_ogv']:
                video_blob = video_data['video_file_ogv']
            elif video_data['video_file_webm']:
                video_blob = video_data['video_file_webm']
            if 'transcript' in video_data:
                data.update({
                    'transcript': video_data['transcript']
                })
            data.update({
                'file': video_blob,
                'subtitle_file': video_data['subtitle_file'],
                'image': video_data['image'],
            })
        return data

    def post_creation(self, obj, pdb_if_exception=False):
        super(VideoType, self).post_creation(obj, pdb_if_exception)
        try:
            from castle.cms.media.video import process
            process(obj)
        except Exception:
            pass


register_import_type('WildcardVideo', VideoType)
register_import_type('video', VideoType)
# register_import_type('youtube-video', VideoType)


class AudioType(FileType):
    def __init__(self, data, path, *args):
        super(FileType, self).__init__(data, path, *args)
        self.data['portal_type'] = 'Audio'
        repo_path = '/audio-repository/'

        if 'retain_paths' in args:
            self.path = path
        else:
            _id = _generate_file_repo_object_id(path)
            self.field_data['id'] = _id
            self.path = repo_path + _id

    def get_data(self):
        data = super(FileType, self).get_data()
        if self.original_type == 'audio':
            data.update({
                'file': self.data['data']['file'],
                'transcript': self.data['data']['transcription']
            })
        elif self.original_type == 'WildcardAudio':
            try:
                transcript = self.data['data']['wildcard.media.behavior.IAudio']['transcript']
            except Exception:
                transcript = ''
            data.update({
                'file': self.data['data']['wildcard.media.behavior.IAudio']['audio_file'],
                'transcript': transcript
            })
        return data


register_import_type('WildcardAudio', AudioType)
register_import_type('audio', AudioType)


class LinkType(BaseImportType):

    youtube = False

    def __init__(self, data, path, *args):
        super(LinkType, self).__init__(data, path, *args)
        url = self.field_data['remoteUrl']
        if 'youtube' in url:
            self.youtube = True
            self.data['portal_type'] = 'Video'
            _id = _generate_file_repo_object_id(path)
            self.path = '/video-repository/' + _id

    def get_path(self):
        if self.data['portal_type'] == 'Video':
            return '/video-repository/' + self.field_data['id']
        return super(LinkType, self).get_path()

    def get_data(self):
        data = super(LinkType, self).get_data()
        if self.youtube:
            data['youtube_url'] = self.field_data['remoteUrl']
        else:
            data['remoteUrl'] = self.field_data['remoteUrl']
        return data


register_import_type('Link', LinkType)


class EventType(BaseImportType):
    fields_mapping = {
        'startDate': 'start',
        'endDate': 'end',
        'eventUrl': 'event_url',
        'contactEmail': 'contact_email',
        'contactName': 'contact_name',
        'contactPhone': 'contact_phone'
    }
    data_converters = {
        'start': DateTime_to_datetime,
        'end': DateTime_to_datetime
    }
    behavior_data_mappers = (
        (IEventBasic, 'start'),
        (IEventBasic, 'end'),
        (IEventBasic, 'whole_day'),
        (IEventBasic, 'open_end'),
        (IEventAttendees, 'attendees'),
        (IEventLocation, 'location'),
        (IEventContact, 'contact_name'),
        (IEventContact, 'contact_email'),
        (IEventContact, 'contact_phone'),
        (IEventContact, 'event_url'),
    )


register_import_type('Event', EventType)


def get_import_type(data, path, *args):
    types = getUtilitiesFor(IImportType)
    _types = {name: klass for name, klass in types}
    if data['portal_type'] in _types:
        return _types[data['portal_type']](data, path, *args)
    logger.info('No explicit mapping for type {type}.'
                        ' Attempting base import type.'
                        .format(type=data['portal_type']))
    return BaseImportType(data, path, *args)



import argparse
import transaction
import tempfile
import logging
import shutil
import os

from zope.component.hooks import setSite
from castle.cms.commands import exiftool
from castle.cms.commands import qpdf
from castle.cms.commands import gs_pdf
from plone.namedfile.file import NamedBlobFile
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot

# PDFs uploaded to CastleCMS 2.5.9 or earlier might have unwanted metadata
# Run this to reprocess them

logger = logging.getLogger('castle.cms')

parser = argparse.ArgumentParser(
    description='...')
parser.add_argument('--site-id', dest='site_id', default='Castle')
parser.add_argument('--all-sites', dest='all_sites', default=False)

args, _ = parser.parse_known_args()


def process_PDFs(site):
    cat = site.portal_catalog
    filebrains = cat(portal_type='File', contentType='application/pdf')
    print('Processing {} PDFs'.format(len(filebrains)))
    count = 0
    for brain in filebrains:
        obj = brain.getObject()
        blob = obj.file.open('r')
        tmp_dir = tempfile.mkdtemp()
        filepath = os.path.join(tmp_dir, obj.file.filename).encode('utf-8').strip()
        file = open(filepath, 'wb')
        file.write(blob.read())
        file.close()
        blob.close()
        exception = False
        try:
            gs_pdf(filepath)
        except Exception:
            logger.warn(u'Could not strip additional metadata with gs {}'.format(filepath), exc_info=True)  # noqa
            exception = True
        if not exception:
            try:
                exiftool(filepath)
            except Exception:
                logger.warn(u'Could not strip metadata with exiftool {}'.format(filepath), exc_info=True)  # noqa
                exception = True
        if not exception:
            try:
                qpdf(filepath)
            except Exception:
                logger.warn(u'Could not strip additional metadata with qpdf {}'.format(filepath), exc_info=True)  # noqa
                exception = True
        if not exception:
            file = open(filepath, 'rb')
            obj.file = NamedBlobFile(file, filename=obj.file.filename)
            file.close()
            count += 1
            if count % 50 == 0:
                print('Processed {} PDFs'.format(count))
                transaction.commit()
        else:
            logger.warn('{sitepath} will not be modified because an exception occured.'.format(sitepath=brain.getURL()))  # noqa
        shutil.rmtree(tmp_dir)
    transaction.commit()
    print('Done.')


if __name__ == '__main__':
    if args.all_sites:
        for oid in app.objectIds():  # noqa
            site = app[oid]  # noqa
            if IPloneSiteRoot.providedBy(site):
                print('Processing site: {}'.format(oid))
                setSite(site)
                process_PDFs(site)
    else:
        site = app[args.site_id]  # noqa
        setSite(site)
        process_PDFs(site)



import ast
from plone.uuid.interfaces import IUUID

from castle.cms.utils import parse_query_from_data
from castle.cms.widgets import QueryFieldWidget
from plone.autoform import directives as form
from plone.schemaeditor.fields import FieldFactory
from plone.schemaeditor.fields import TextLineChoiceField
from plone.schemaeditor.interfaces import IFieldEditFormSchema
from Products.CMFCore.utils import getToolByName
from zope import schema
from zope.component import adapter
from zope.globalrequest import getRequest
from zope.i18nmessageid import MessageFactory
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.schema.interfaces import IChoice
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.interfaces import IField
from zope.schema.vocabulary import SimpleVocabulary


_ = MessageFactory('castle.cms')


class IQueryChoice(IChoice):
    query = schema.List(
        title=u'Query',
        value_type=schema.Dict(value_type=schema.Field(),
                               key_type=schema.TextLine()),
        required=False
    )

    sort_on = schema.TextLine(
        title=u'Sort on',
        required=False,
        default=u'effective'
    )

    sort_reversed = schema.Bool(
        title=u'Reversed order',
        required=False,
        default=True
    )

    limit = schema.Int(
        title=u'Limit',
        required=False,
        default=15,
        min=1,
    )


@implementer(IQueryChoice)
class QueryChoice(schema.Choice):

    def __init__(self, query=None, sort_on='effective', sort_reversed=True,
                 limit=15, **kw):
        self.query = query or []
        self.sort_on = sort_on
        self.sort_reversed = sort_reversed
        self.limit = limit
        kw['source'] = QueryChoiceSource(self)  # bind to field
        self._fix_query()
        super(QueryChoice, self).__init__(**kw)

    def _fix_query(self):
        # does not save data properly often here
        for item in (self.query or []):
            if (item['v'] and
                    isinstance(item['v'], basestring) and
                    item['v'][0] in ('[', '{') and
                    item['v'][-1] in (']', '}')):
                item['v'] = ast.literal_eval(item['v'])


@implementer(IContextSourceBinder)
class QueryChoiceSource(object):
    __name__ = 'QueryChoiceSource'

    def __init__(self, field):
        self.field = field

    def get_cache_key(self, context):
        return 'easyform-source-value--{}-{}'.format(
            IUUID(context, 'default'), self.field.__name__)

    def __call__(self, context):
        cache_key = self.get_cache_key(context)
        request = getRequest()
        if request is not None and cache_key in request.environ:
            # prevent doing query many times
            return request.environ[cache_key]

        if not self.field.query:
            return SimpleVocabulary([])

        query = parse_query_from_data({
            'query': self.field.query,
            'sort_on': self.field.sort_on,
            'sort_reversed': self.field.sort_reversed,
        })
        catalog = getToolByName(context, 'portal_catalog')
        if query.get('sort_on', '') not in catalog._catalog.indexes:
            query['sort_on'] = 'effective'

        terms = []
        for item in catalog(**query)[:self.field.limit]:
            terms.append(
                SimpleVocabulary.createTerm(item.id, item.id, item.Title))

        value = SimpleVocabulary(terms)
        if request is not None:
            request.environ[cache_key] = value
        return value


# so we can still resolve correctly
alsoProvides(QueryChoiceSource, IContextSourceBinder)


QueryChoiceFactory = FieldFactory(
    QueryChoice, _(u'label_query_choice_field', default=u'Query Choice'),
    source=QueryChoiceSource)


class IQueryChoiceFieldSchema(IField):
    form.widget(query=QueryFieldWidget)
    query = schema.List(
        title=u'Query',
        value_type=schema.Dict(value_type=schema.Field(),
                               key_type=schema.TextLine()),
        required=False
    )

    sort_on = schema.TextLine(
        title=u'Sort on',
        required=False,
        default=u'effective'
    )

    sort_reversed = schema.Bool(
        title=u'Reversed order',
        required=False,
        default=True
    )

    limit = schema.Int(
        title=u'Limit',
        required=False,
        default=15,
        min=1,
    )


@implementer(IFieldEditFormSchema)
@adapter(IQueryChoice)
def getQueryChoiceFieldSchema(field):
    return IQueryChoiceFieldSchema


@implementer(IQueryChoiceFieldSchema)
@adapter(IQueryChoice)
class TextLineQueryChoiceField(TextLineChoiceField):
    pass



from plone import api
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.resources.browser.cook import cookWhenChangingSettings


PROFILE_ID = 'profile-castle.cms:2_3_1'


def upgrade(context, logger=None):
    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile(PROFILE_ID)
    cookWhenChangingSettings(api.portal.get())

    setup.runImportStepFromProfile(
        'profile-collective.elasticsearch:default',
        'plone.app.registry',
        run_dependencies=False,
    )



from castle.cms import trash
from castle.cms.interfaces import ITrashed
from castle.cms.constants import TRASH_LOG_KEY
from plone import api
from plone.app.linkintegrity.exceptions import LinkIntegrityNotificationException
from plone.locking.interfaces import ILockable
from zope.annotation.interfaces import IAnnotations
from Products.Five import BrowserView
from unidecode import unidecode
from zope.event import notify
from castle.cms.events import TrashEmptiedEvent


class TrashView(BrowserView):

    def __call__(self):
        self.catalog = api.portal.get_tool('portal_catalog')
        self.site = api.portal.get()
        self.site_path = '/'.join(self.site.getPhysicalPath())
        if self.request.REQUEST_METHOD == 'POST':
            action = self.request.get('action')
            if action == 'restore':
                self.restore()
            elif action == 'delete':
                self.delete()
            elif action == 'empty':
                self.empty()
        self.items = self.catalog(trashed=True, sort_on='modified',
                                  sort_order='reverse',
                                  object_provides=ITrashed.__identifier__)
        return self.index()

    def get_label(self, brain):
        label = brain.Title
        if brain.is_folderish:
            num = len(self.catalog(trashed=True,
                                   path={'query': brain.getPath(), 'depth': -1}))
            label += '(%i items)' % num
        return label

    def get_path(self, brain):
        if hasattr(brain, 'getPath'):
            path = brain.getPath()
        else:
            path = '/'.join(brain.getPhysicalPath())
        return path[len(self.site_path):]

    def get_by_uid(self, uid):
        return self.catalog(UID=uid, trashed=True)[0].getObject()

    def restore(self):
        uid = self.request.get('uid')
        obj = self.get_by_uid(uid)
        api.portal.show_message(u'Successfully restored "%s" located at: %s' % (
            unidecode(obj.Title()), self.get_path(obj)), self.request, type='info')
        trash.restore(obj)

    def delete(self):
        obj = self.get_by_uid(self.request.get('uid'))
        lockable = ILockable(obj, None)
        if lockable and lockable.locked():
            lockable.clear_locks()
        try:
            api.content.delete(obj)
            api.portal.show_message('Successfully deleted "%s" located at: %s' % (
                unidecode(obj.Title()), self.get_path(obj)), self.request, type='warning')
        except LinkIntegrityNotificationException:
            api.portal.show_message('Can not delete "%s" located at: %s because it is still linked.' % (
                unidecode(obj.Title()), self.get_path(obj)), self.request, type='warning')

    def empty(self):
        for item in [i for i in self.catalog(trashed=True,
                                             object_provides=ITrashed.__identifier__)]:
            obj = item.getObject()
            lockable = ILockable(obj, None)
            if lockable and lockable.locked():
                lockable.clear_locks()
            try:
                api.content.delete(obj, check_linkintegrity=False)
            except LinkIntegrityNotificationException:
                # could be a folder that has been deleted
                api.portal.show_message(
                    'Some content could not be removed because it is still linked '
                    'to other content on the site.',
                    self.request, type='warning')
        notify(TrashEmptiedEvent(self))
        api.portal.show_message('Trash emptied', self.request, type='warning')

    def get_trash_log(self):
        annotations = IAnnotations(self.site)
        if TRASH_LOG_KEY in annotations:
            return annotations[TRASH_LOG_KEY]
        else:
            return "No empty operation log, the trash has not been emptied"



import unittest
import os
import robotsuite
from castle.cms.testing import CASTLE_ROBOT_TESTING
from plone.testing import layered


dir_path = os.path.dirname(os.path.realpath(__file__))


def test_suite():
    suite = unittest.TestSuite()

    for filename in os.listdir(os.path.join(dir_path, 'robot')):
        if (not filename.startswith('test_') or
                not filename.endswith('.robot')):
            continue
        filepath = os.path.join('robot', filename)
        suite.addTests([
            layered(robotsuite.RobotTestSuite(filepath),
                    layer=CASTLE_ROBOT_TESTING),
        ])
    return suite



import argparse

import transaction
from AccessControl.SecurityManagement import newSecurityManager
from castle.cms.cron.utils import setup_site
from castle.cms.utils import retriable
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot


parser = argparse.ArgumentParser(
    description='...')
parser.add_argument('--site-id', dest='site_id', default=None)
parser.add_argument('--skip-incomplete', dest='skip_incomplete', default=False,
                    action='store_true')
args, _ = parser.parse_known_args()


@retriable(sync=True)
def upgrade(site):
    setup_site(site)

    # attempt to upgrade plone first
    pm = site.portal_migration
    report = pm.upgrade(dry_run=False)
    print(report)

    ps = site.portal_setup
    # go through all profiles that need upgrading
    for profile_id in ps.listProfilesWithUpgrades():
        # do our best to detect good upgrades
        if profile_id.split(':')[0] in (
                'Products.CMFPlacefulWorkflow', 'plone.app.iterate',
                'plone.app.multilingual', 'Products.PloneKeywordManager',
                'collective.easyform', 'plone.session'):
            continue
        if not profile_id.endswith(':default'):
            continue
        steps_to_run = ps.listUpgrades(profile_id)
        if steps_to_run:
            print('Running profile upgrades for {}'.format(profile_id))
            ps.upgradeProfile(profile_id)

        remaining = ps.listUpgrades(profile_id)
        if remaining:
            if args.skip_incomplete:
                print(
                    '[{}] Running upgrades did not finish all upgrade steps: {}'.format(
                        profile_id, remaining))
            else:
                raise Exception(
                    '[{}] Running upgrades did not finish all upgrade steps: {}'.format(
                        profile_id, remaining))

    transaction.commit()


def run(app):
    user = app.acl_users.getUser('admin')
    newSecurityManager(None, user.__of__(app.acl_users))

    if args.site_id:
        upgrade(app[args.site_id])
    else:
        for oid in app.objectIds():
            obj = app[oid]
            if IPloneSiteRoot.providedBy(obj):
                upgrade(obj)


if __name__ == '__main__':
    run(app)  # noqa



from zope.component import getUtility
from plone.registry.interfaces import IRegistry


_recover_slot_tiles = {
    "Media": [
        "castle.cms.audiotile",
        "castle.cms.embedtile",
        "castle.cms.maptile",
        "castle.cms.gallerytile",
        "castle.cms.slidertile",
        "castle.cms.videotile",
        "castle.cms.imagetile"
    ],
    "Social": [
        "castle.cms.facebookPage",
        "castle.cms.pin",
        "castle.cms.twitterTimeline",
        "castle.cms.tweet",
        "castle.cms.sharing"
    ],
    "Advanced": [
        "castle.cms.fragment",
        "castle.cms.calendar",
        "castle.cms.survey",
        "castle.cms.querylisting",
        "plone.app.standardtiles.contentlisting",
        "castle.cms.navigation",
        "castle.cms.search"
    ]
}


def upgrade(context, logger=None):
    registry = getUtility(IRegistry)
    if registry['castle.slot_tiles'] is None:
        registry['castle.slot_tiles'] = {}
    tiles = registry['castle.slot_tiles']
    for recover_type, recover_tiles in _recover_slot_tiles.items():
        if recover_type not in tiles:
            tiles[recover_type] = []
        for tile_id in recover_tiles:
            if tile_id not in tiles[recover_type]:
                tiles[recover_type].append(tile_id)



from castle.cms import cache
from plone import api
from plone.protect.authenticator import createToken
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


def get_backend_url():
    registry = getUtility(IRegistry)
    backend_url = registry.get('plone.backend_url', None)
    if not backend_url or len(backend_url) == 0:
        return api.portal.get().absolute_url()
    else:
        return backend_url[0]


def no_backend_url(value):
    try:
        url = str(value)
        backend = get_backend_url()
        if backend in url:
            return False
        return True
    except Exception:
        return True


def get_public_url():
    public_url = api.portal.get_registry_record('plone.public_url')
    if not public_url:
        public_url = api.portal.get().absolute_url()
    return public_url


def site_has_icon():
    key = '{}.has_site_icon'.format(
        ''.join(api.portal.get().getPhysicalPath()))
    try:
        has_icon = cache.ram.get(key)
    except KeyError:
        registry = getUtility(IRegistry)
        has_icon = False
        try:
            has_icon = bool(registry['plone.site_icon'])
        except Exception:
            pass
        cache.ram.set(key, has_icon)
    return has_icon


def get_upload_fields(registry=None):
    if registry is None:
        registry = getUtility(IRegistry)
    upload_fields = registry.get('castle.file_upload_fields', None)
    if upload_fields is None:
        # not updated yet, b/w compatiable
        required_upload_fields = registry.get(
            'castle.required_file_upload_fields', []) or []
        result = [{
            'name': 'title',
            'label': 'Title',
            'widget': 'text',
            'required': 'title' in required_upload_fields,
            'for-file-types': '*'
        }, {
            'name': 'description',
            'label': 'Summary',
            'widget': 'textarea',
            'required': 'description' in required_upload_fields,
            'for-file-types': '*'
        }, {
            'name': 'tags',
            'label': 'Tags',
            'widget': 'tags',
            'required': 'tags' in required_upload_fields,
            'for-file-types': '*'
        }, {
            'name': 'youtube_url',
            'label': 'Youtube URL',
            'widget': 'text',
            'required': 'youtube_url' in required_upload_fields,
            'for-file-types': 'video'
        }]
    else:
        result = []
        for field in upload_fields:
            if 'name' not in field:
                continue
            # need to make copy of data otherwise we're potentially
            # modifying the record directly
            data = {}
            data.update(field)
            # make sure all required field are in place
            if data.get('required'):
                data['required'] = str(data['required']).lower() in ('true', 't', '1')
            else:
                data['required'] = False
            if 'label' not in data:
                data[u'label'] = data[u'name'].capitalize()
            if 'widget' not in field:
                data[u'widget'] = u'text'
            if 'for-file-types' not in data:
                data[u'for-file-types'] = u'*'
            result.append(data)
    return result


def get_chat_info():

    try:
        frontpage = api.portal.get_registry_record('castle.rocket_chat_front_page')
        salt = api.portal.get_registry_record('castle.rocket_chat_secret')
    except api.exc.InvalidParameterError:
        frontpage = None
        salt = ''

    if frontpage is None or salt == '':
        return

    if frontpage[-1] != '/':
        frontpage = frontpage + '/'

    url = frontpage.replace('http://', 'ws://')
    url = url + 'websocket'

    current = api.user.get_current()
    base_url = api.portal.get().absolute_url()

    return {
        'url': url,
        'base_url': base_url,
        'frontpage': frontpage,
        'token': createToken(salt),
        'user': getattr(current, 'id', ''),
        'email': current.getProperty('email')
    }



from AccessControl import Unauthorized
from Acquisition import aq_base, aq_parent
from castle.cms.behaviors.location import ILocation
from castle.cms.interfaces import (IHasDefaultImage, IReferenceNamedImage,
                                   ITrashed)
from collective.elasticsearch.interfaces import IReindexActive
from OFS.interfaces import IItem
from plone import api
from plone.app.uuid.utils import uuidToCatalogBrain as get_brain
from plone.app.contenttypes.interfaces import IFile, IImage
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer.decorator import indexer
from plone.uuid.interfaces import IUUID
from ZODB.POSException import POSKeyError
from zope.globalrequest import getRequest
from plone.event.interfaces import IEvent
from Products.CMFCore.interfaces import ISiteRoot


@indexer(IItem)
def getRawRelatedItems(obj):
    try:
        result = []
        for relation in obj.relatedItems:
            try:
                to_obj = relation.to_object
                if to_obj:
                    uuid = IUUID(to_obj)
                    if uuid:
                        result.append(uuid)
            except AttributeError:
                pass
        return result
    except AttributeError:
        return []


@indexer(IItem)
def getLocation(obj):
    bdata = ILocation(obj, None)
    if bdata and bdata.location:
        return bdata.location[0]


@indexer(IItem)
def hasImage(obj):
    if IHasDefaultImage.providedBy(obj):
        return True
    return getattr(aq_base(obj), 'image', None) is not None


@indexer(IItem)
def image_info(obj):
    try:
        image = obj.image
        if IReferenceNamedImage.providedBy(image):
            data = {'reference': image.reference}
        else:
            width, height = image.getImageSize()
            data = {
                'width': width,
                'height': height,
                'focal_point': [width / 2, height / 2]
            }

            try:
                data['focal_point'] = image.focal_point
            except AttributeError:
                try:
                    data['focal_point'] = obj._image_focal_point
                except Exception:
                    pass
        return data
    except AttributeError:
        pass


@indexer(IItem)
def getContentTypeContent(obj):
    return 'text/html'


@indexer(IFile)
def getContentTypeFile(obj):
    try:
        return obj.file.original_content_type
    except Exception:
        try:
            return obj.file.contentType
        except Exception:
            pass


@indexer(IImage)
def getContentTypeImage(obj):
    try:
        return obj.image.contentType
    except Exception:
        pass


@indexer(IEvent)
def recurrence(obj):
    try:
        return obj.recurrence
    except AttributeError:
        pass


@indexer(IItem)
def trashed(obj):
    while obj:
        if ITrashed.providedBy(obj):
            return True
        obj = aq_parent(obj)
    return False


@indexer(IDexterityContent)
def last_modified_by(context):
    req = getRequest()
    creator = getattr(aq_base(context), '_last_modified_by', None)
    if req is not None and not IReindexActive.providedBy(req):
        # here, we're assuming the current user editing the object
        # is the logged in user performing the current action
        creator = api.user.get_current().getId()
        # also set on context so we can retrieve later when reindexing
        context._last_modified_by = creator

    if creator is None:
        try:
            creator = context.Creators()[0]
        except Exception:
            creator = 'admin'

    rt = api.portal.get_tool("portal_repository")

    if rt is None or not rt.isVersionable(context):
        # not versionable; fallback to the creator
        return creator

    try:
        history = rt.getHistoryMetadata(context)
    except Unauthorized:
        history = None
    if not history:
        return creator

    try:
        if not rt.isUpToDate(context):
            return creator
    except POSKeyError:
        return creator

    length = history.getLength(countPurged=False)

    last = history.retrieve(length - 1)
    if not last or type(last) != dict:
        # unexpected version metadata; fallback to the creator
        return creator

    metadata = last.get("metadata")
    if not metadata or type(metadata) != dict:
        # unexpected version metadata; fallback to the creator
        return creator

    sys_metadata = metadata.get("sys_metadata")
    if not sys_metadata or type(sys_metadata) != dict:
        # unexpected version metadata; fallback to the creator
        return creator

    principal = sys_metadata.get("principal")
    if not principal or type(principal) != str:
        # unexpected version metadata; fallback to the creator
        return creator

    return principal


@indexer(IItem)
def has_private_parents(obj):
    if (api.content.get_state(obj) != 'published'):
        return True  # needs to be True for private self as well as parents
    parent = aq_parent(obj)
    while not ISiteRoot.providedBy(parent):
        try:
            parent_brain = get_brain(parent.UID())
            try:
                if parent_brain.has_private_parents:
                    return True
            except AttributeError:
                if api.content.get_state(parent) != 'published':
                    return True
        except Exception:
            pass  # to be extra secure, could return True here. Better to be fault tolerant for now.
        parent = aq_parent(parent)
    return False



from castle.cms import constants
from castle.cms.interfaces import ISecureLoginAllowedView
from plone import api
from plone.registry.interfaces import IRegistry
from zExceptions import Redirect
from zope.component import queryUtility
from Acquisition import aq_parent

SHIELD = constants.SHIELD

_blacklisted_meta_types = (
    'Image', 'File', 'Filesystem Image',
    'Filesystem File', 'Stylesheets Registry', 'JavaScripts Registry',
    'DirectoryViewSurrogate', 'KSS Registry', 'Filesystem Directory View')


def protect(req, recheck=False):
    url = req.getURL()
    login_url = '{}/@@secure-login'.format(api.portal.get().absolute_url())
    if '@@secure-login' in url.lower() and url != login_url:
        raise Redirect(login_url)

    published = req.PARENTS[0]
    mt = getattr(
        getattr(published, 'aq_base', None),
        'meta_type',
        getattr(published, 'meta_type', None))
    if mt in _blacklisted_meta_types or mt is None:
        return

    published = req.get('PUBLISHED')
    if ISecureLoginAllowedView.providedBy(published):
        return

    registry = queryUtility(IRegistry)
    setting = (registry and
               registry.get('plone.login_shield_setting', SHIELD.NONE) or
               SHIELD.NONE)

    protect = False
    if setting == SHIELD.ALL:
        protect = True
    elif setting == SHIELD.BACKEND:
        backend_urls = (registry and
                        registry.get('plone.backend_url', SHIELD.NONE) or
                        [])
        for backend_url in backend_urls or []:
            try:
                protect |= backend_url.startswith(req.SERVER_URL)
            except AttributeError:
                pass
    if protect:
        if req.getURL().lower().endswith("robots.txt"):
            return """User-agent: *
Disallow: /"""

        if recheck:
            portal = api.portal.get()
            site_plugin = portal.acl_users.session
            creds = site_plugin.extractCredentials(req)
            anonymous = not site_plugin.authenticateCredentials(creds)
            if anonymous:
                try:
                    app_plugin = aq_parent(portal).acl_users.session
                    anonymous = not app_plugin.authenticateCredentials(creds)
                except AttributeError:
                    anonymous = True
        else:
            anonymous = api.user.is_anonymous()

        if anonymous:
            raise Redirect(login_url)



from castle.cms.tiles.base import BaseTile
from plone.supermodel import model
from zope import schema
from zope.schema.vocabulary import SimpleVocabulary


class TwitterTimelineTile(BaseTile):
    def render(self):
        self.screenName = self.data.get('screenName', '')

        if self.screenName is None:
            return

        if self.screenName[0] == '@':
            self.sreenName = self.screenName[1:]

        self.height = self.data.get('height', '400')
        self.theme = self.data.get('theme', 'light')
        self.embed_url = 'https://twitter.com/{screenname}?ref_src=twsrc%5Etfw'.format(
            screenname=self.screenName)
        self.portalUrl = self.context.portal_url()

        return self.index()


class ITwitterTimelineTileSchema(model.Schema):
    screenName = schema.TextLine(
        title=u"Screen Name",
        description=u"Screen name of the user whose timeline you want to embed."
    )

    height = schema.Int(
        title=u"Height",
        description=u"Height of the rendered timeline, in pixels.",
        default=400,
        min=200,
        required=False
    )

    theme = schema.Choice(
        title=u"Theme",
        description=u"Timeline color scheme.",
        vocabulary=SimpleVocabulary.fromValues(['light', 'dark']),
        default='light',
        required=False
    )



# -*- coding: utf-8 -*-
from plone.registry.interfaces import IRegistry
from plone.namedfile.interfaces import INamedFile
from zExceptions import NotFound
from zope.component import getUtility
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import setRoles
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone.formwidget.namedfile.converter import b64encode_file
from castle.cms.browser.files.icon import IconView
import unittest


bimage = b'GIF89a=\x00D\x00\xf7\xa8\x00\x9a,3\xff\xc0\xc0\xef\xc0\xc0uXg\xfc\xf9\xf7\x993\x00\xff\xec\xec\xff\xa0\xa0\xe5\xcc\xbf\xcf\x9f\x87\x0f\xef\xef\x7f\x7f\x7f\xef\x0f\x0f\xdf\x1f\x1f\xff&&_\x9f\x9f\xffYY\xbf??5\xa5\xc2\xff\xff\xff\xac\x16\x19\xb2&\x00\xf8\x13\x10\xc2& \xdf`PP\x84\x9b\xf8\x03\x00\xb5\x0b\x0c\xdf\x0f\x00>\x9a\xb5\x87BM\x7f`P\xd2\xa5\x8f\xcc\x19\x00\xa5,\x00\xec\xd9\xcf\xe5\x0c\x00\xeb\t\x00\xff\xd9\xd9\xc7\x0c\x0c\x0f\x0f\x0f\xffyy~MZ\xfb\t\x08\xe5M@\xfb__\xff33\xcf\x90x\xf2\xe5\xdf\xc3\x06\x06\xbf\t\x08\xff\xb3\xb3\xd9\xb2\x9f\xff\x06\x06\xac)\x00\xff\xc6\xc6\x0c\t\x08\xf9\xf2\xef\xc9s`\xb8#\x00\x9f/\x00\xff__\xff\x8c\x8c\xc5\x1c\x00\xdf33\xffpp\xcf\x19\x19\xc0\x13\x10\xbf\x90x\xf7YY\xff\xf6\xf6\xe7??\xd7&&\xefLL2& \xdf\xbf\xaf\xbf\xbf\xbf???\xc5M@cn\x81_\x00\x00___\xcb00\xd8\x13\x00YC8\x80\x80\x80\xf3RRsVH\xc490\x10\x10\x10\x917@\xf2\x06\x00\xcf@@\xca\x86pooo\xa3!&\xc1\x1d\x18\xcf//\x1f\x1f\x1f\xdf\x00\x00\xd2\x16\x00\xcb\x90x\xbf\x1f\x00\x19\x13\x10\xf3\xd0\xd0\xe399&\x1d\x18Yy\x8e\x8f\x8f\x8f\xff\xa9\xa9\xcb\x13\x13\xbf00SF@\xb6& >\x1d\x18\xfb\xdd\xdd@@@\x99\x93\x90\xff\xbc\xbc\x7fPP\xaf\xaf\xaf\xc6VHzsp\x93& \xb7pp\xb3\x86ptPP|pp\xafOO\xd0\xd0\xd0\xef\xef\xefL90\xbc\xa9\xa0o0(\xeb\xb0\xb0\xff\xe0\xe0\xff\xd0\xd0\x870(K0(\xc9|h\x9f__lct\xebFF\xcf\xcf\xcf\xe0\xe0\xe0b& \xff  },(@0(\xa9\x93\x88\xa6|h\x1f\xdf\xdf\xd5\xac\x97\xe2\xc5\xb7\xc7`POOO\x9cyhppp\xff\x80\x80\xff\x96\x96\xd7``\xcc\x99\x7f,\xb0\xcf\xbf\x00\x00\x00\x00\x00\x00\xff\xff\xff\x00\x00\xffff\xff\xff\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00!\xf9\x04\x01\x00\x00\xa8\x00,\x00\x00\x00\x00=\x00D\x00\x00\x08\xff\x00Q\t\x1cH\xb0\xa0\xc1\x83\x08\x13*\\\xc8\xb0\xa1\xc0\x1b\x07\x0c8\x9cHq\xa1\x89\x14\xa72F\xac\xc8\xb1\xa2\t\x1f\x19Cj\x94\xd8\xb1$B\x03\x07D\xaa\x1ci\xb2%*#3V\xcad\xe9\xb2\xa2\x9d 3s\x9e\xdaX\x93!"\x8c:\x83\xf2\xeci\xf0c\xd0\xa3!\x87\x12E\x89\xb4iR\x92.a:\x9d\xfa\xb4\xe5\x0c\x9cT\xb3\xee\x84:\xf1\x06P\xad`\x95*4\n\xb6l\xd5\x84\x06>\x99]\x1b\xb2\xc5\x9c\x83F\xda\xb0\x9d{\xe4\x84\x00\x83W\xe7\xaeM\xe2f\xd4\xa8\xbb\x03\xbd\xea5kE\x88_\xbf\x80\x0fy\x1a\\\xb6\x08\x92\xc3\x87\x01\x070\xe5\x00\x02\xe3\xa9-\x80\xc4\x80\x1cY\xe0dS\x94-_\x0ezd3\xe7\xce\xa8>\x83\x0e=Zf\x92\x13\xa7Gm\x18 \xe1\xaf\xe7\xd5\xb8+\xb7\xceX8\xf6(\xda\xa2D\xd9N\x8d\xbb\xb8n\xc6\x8e}\x8f\xfa\x12<\xf8\xf0\xcf\x11\x1a\x14\x07}|mf\xdf\x00\x9elP\xd1\\\xb8dSaJ\x95\xffz }zu\xadiLs\xa6\xb0&8\x80\x01\xdd\x9f\x9b\x8a ^<\xf9\xe9\xac\xa9:\x82\x1d{\x83\x84\xe6\xef\xc5\xf7\x1d}\xf5\xd9W\x9eq\xa2\x1d\x95\x84a\xb1\xa9\xb0\x01\x00\xdd\x05\xd8\x9c|\x04\x16X\x8a\x02\x0b0\x80\x9f\x0b=\xe8\x94\\l\x1et \n\x00\x10\x02\x08\xdf\x84\x03ZX \x86\x1a\x16W\x03\x87+]\xe7[\x06\x00\x96\xe8\xde\x89\xce\xa5\xa8\xe2\x8a\x19N\xf7b\x87\x19\xa5\x17\x1b\x05\xa3P\x10\xa1\x8d#\xe2X\x9b\x8e;\xf2\xd8"n/\xd6\xd5\xdf\x13\xa2x\x80$\x89\x11\x9e\xd8\x81\x16\x146\xb9#\x8b\xd3\xf9\xe6\xc1\x7f\xa2\x0cp\xe5\x99\x12\xa8\x80\xdad\x15zi!\x98\xab\xf9Ff\x99gvG$g\xdf1\xa0\x80\x9bM\xc2\t\x19\x00\x19p\xd9\x9d\x99G6\xd7Hl\xdf\x99\xc2\xc8\x9e|~\t\x88)~Q@c\x99\xa3\x0cZg\x06\x00\xf8\x96\xa8)\x0c,\xc0h\xa3\x05^\x02\xe9(\x93Rji\x84\xcb)\'\x1fn\x9d~\nj)\xa3\x0e\xffZis\x84\x06\xd7\x81\xaak\xae\xc6\x01\x07\xa0\xb5\xfa*\xac~\xc9z\xaa\x04\x03l\x80+b\xb7\x81V@\x01$\xac\xd6\xe9\xab\xb1\xd2:kpj\x0ep\xe7\xb1\xab\x9aRA\x01!\x14\xd7\xc0\x03\x8dF\x1b\xdc\x00\xd3\x8ar-\xb6\xc8\x12\x07Z\t\x15\xf0:\xdd\xb7n\x8ak\xaa(\x1ddz\xac\x14\x86\x80\x92+~\xf8\xc1\xbb\xa3\xbc\xe4\xae\xe1\x01\xbaR\xfcAG\'\\\xa4\xab\x1a\xbf\xef\x82k\xa1\xbc\x03\xa3\xeb\xd7\x1d\xa4T\xcc\x87\xc2\xc5qP\x02\xc3\xab\xf9+\x9e\xb8OH\xec\xd7\x1bYTL\x8a\x1f~\xa1\x91\xecj"\xd8\xc01n\xfe\x8e\xdaA\x06\xe7\xa2;\t)Q\xb0AJ\x15\\\xa8\xbc2h!\x14\xe0\xee\xcb\xa05\x10\xc6\xa8"s&\x07\n\x13L\xb0sA\x0b\x9b\xa2\x81\x08"h\xf02\x0f\x15\xe0\x964g2\xa8\xd1D\xd3\xa4\xe8\x01\xf5t\x1c\x14`\xc6\xcb\xcbN\x11\xe7\xd6\x87]@\xca\xd7\x8f\x90\xf2\x01\x08#\x10t\x80$\xc5\x99\xc1-\xc7?\x14\xff@\xc6\xdal\x8f\xe2\x04)b0\xb1\t)}\x84\x12J&\x04\x05\x02\xc5\x18\xb8\xd9P\xc0\x0f\x1c\x93`5h\x81_\xb0H(j\x98\xacD( \xc0`P\xc5\x8f\x83\xa6\xc1\xb6;l1\x9d\x06\x1bk\x9d4\x18:(\x1e\n\x15&sR\xb7A9\xc0Q\xf1 \x18X\x00Z\xdf<\x84\xa0:h$H^\x1cgC\\\xa0\xdc\x10\x9a\xc8\xae8\x11gdQ\x07\x01\x07!\x10\n\x11W| {\xef\xa6\x90\xb0m\x01"T B\x01<\xa8\xed\xba_X|pE\x1e\xa7\xc9\xe0D\x19\xce\xcb\xbe\x04\xf5\x08\x11\x80@\x02\xf1+\xce}\t!\xecP\xc1\x0ed\xb8\xdc\xf9\x86\xa0\x88\x8aQA\x06\x90\xc1\x02\xfc\xf2G\x83\x1c4\xc4~\xf8\xcb\x1f\xf7^v\x98D\x98\x0c\x07\xca\x1b\xc5\x05\xba\x90\xbfP`Bt\x14\x81`\x07\'\xc8/\xbf\xc8@\toC\x01)\x9c\x00\xbb\x0e\xd2\xcd$"\x94\xa0\xef\xf0\xe3\x978\xe0l\x02^ \x05\x07\xf3\x97\x00\x04\xd0\xaf%1t\xde\x0b|X\xb0\x820\x8db\x0f\xa4`\xc2\x04\x16@\x8a\x0e\xce\x8f(\x02\t\xa2\xec\x86X\xc4\xb5\x15"\x898\xc4A\xfc\x1a\x08\xc5\x82HQqT\xc4\xdc("A\n<\x08\x02\x05\x94\x90\x1d\r@\xd8E\x83|1\x14T\xbc\x80\x0e>@\n\x14\x88An\xa0\xbb]\x1b\x13\xf2F\xd9Y\xc2dg\xe8\xe1\x1e\x1d\xd2\xc7P\xa0\x10\x07\x84\xf8\xe1 \x1fx\xbf\xfc\x11\xa1\x12\x90XdG\x82\xb8FI\x02q\t/\xb4\xa4&[\x12\x10\x00;'  # noqa


class TestSiteIcon(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_get_site_icon_does_not_raise_overflow_error(self):
        registry = getUtility(IRegistry)
        registry['plone.site_icon'] = b64encode_file('foobar.gif', bimage)
        view = IconView(self.portal, self.request)
        self.request.form.update({
            'scale': '-3284239847329'
        })
        self.assertRaises(NotFound, view._getFile)

    def test_get_site_icon_raises_404(self):
        registry = getUtility(IRegistry)
        registry['plone.site_icon'] = b64encode_file('foobar.gif', bimage)
        view = IconView(self.portal, self.request)
        self.request.form.update({
            'scale': '555'
        })
        self.assertRaises(NotFound, view._getFile)

    def test_get_site_icon(self):
        registry = getUtility(IRegistry)
        registry['plone.site_icon'] = b64encode_file('foobar.gif', bimage)
        view = IconView(self.portal, self.request)
        self.request.form.update({
            'scale': '180'
        })
        result = view._getFile()
        self.assertTrue(INamedFile.providedBy(result))



import os

import httplib2
from apiclient.discovery import build
from oauth2client.client import OAuth2Credentials
from oauth2client.client import SignedJwtAssertionCredentials


GOOGLE_CLIENT_ID = os.environ.get('GOOGLE_CLIENT_ID')
GOOGLE_CLIENT_SECRET = os.environ.get('GOOGLE_CLIENT_SECRET')


def get_service(api_name, api_version, scope, key=None,
                service_account_email=None, access_token=None,
                refresh_token=None, token_expiry=None):
    """Get a service that communicates to a Google API.

    Args:
        api_name: The name of the api to connect to.
        api_version: The api version to connect to.
        scope: A list auth scopes to authorize for the application.
        key_file_location: The path to a valid service account p12 key file.
        service_account_email: The service account email address.

    Returns:
        A service that is connected to the specified API.
    """
    service = None

    if key and service_account_email:
        credentials = SignedJwtAssertionCredentials(
            service_account_email, key, scope=scope)

        http = credentials.authorize(httplib2.Http())

        # Build the service object.
        service = build(api_name, api_version, http=http)
    elif GOOGLE_CLIENT_ID and GOOGLE_CLIENT_SECRET:
        creds = OAuth2Credentials(
            access_token=access_token, client_id=GOOGLE_CLIENT_ID,
            client_secret=GOOGLE_CLIENT_SECRET, refresh_token=refresh_token,
            token_uri='https://accounts.google.com/o/oauth2/token',
            token_expiry=token_expiry, user_agent='Castle CMS')
        http = creds.authorize(httplib2.Http())
        service = build(api_name, api_version, http=http)

    return service



from castle.cms.tiles.base import BaseTile
from plone.supermodel import model
from zope import schema
from zope.schema.vocabulary import SimpleVocabulary

import re


class TweetTile(BaseTile):
    def render(self):
        content = self.data.get('url', '') or ''
        self.theme = self.data.get('theme', 'light')

        urlRegEx = ".*twitter\.com/.*/status/([0-9]+)"
        urlParser = re.compile(urlRegEx)
        matches = urlParser.match(content)

        if matches is None:
            return ''

        self.embed_url = '{content}?ref_src=twsrc%5Etfw'.format(content=content)

        self.portalUrl = self.context.portal_url()

        return self.index()


class ITweetTileSchema(model.Schema):
    url = schema.URI(
        title=u"URL",
        description=u"URL to the tweet to include in the tile."
    )

    theme = schema.Choice(
        title=u"Theme",
        description=u"Timeline color scheme.",
        vocabulary=SimpleVocabulary.fromValues(['light', 'dark']),
        default='light',
        required=False
    )



# -*- coding: utf-8 -*-
import json
import unittest
from copy import deepcopy
from io import BytesIO

from castle.cms.browser import content
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.interfaces.syndication import IFeedSettings
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility


class TestContent(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_dump_data(self):
        doc = api.content.create(
            type='Document',
            id='doc',
            container=self.portal)
        data = content.dump_object_data(doc)
        self.assertEqual(json.loads(data)['uid'], IUUID(doc))

    def test_dump_data_with_view(self):
        folder = api.content.create(
            type='Folder',
            id='file-repository',
            container=self.portal)
        doc = api.content.create(
            type='File',
            id='fi',
            container=folder)
        data = content.dump_object_data(doc)
        self.assertEqual(json.loads(data)['url'], doc.absolute_url() + '/view')

    def test_workflow_transition(self):
        doc = api.content.create(
            type='Document',
            id='doc',
            container=self.portal)
        self.request.form.update({
            'transition_id': 'publish',
            'action': 'transition'
        })
        wf = content.Workflow(doc, self.request)
        wf()
        self.assertEqual(api.content.get_state(doc), 'published')

    def test_publish_content_twice(self):
        doc = api.content.create(
            type='Document',
            id='doc',
            container=self.portal)
        pc = content.PublishContent(doc, self.request)
        pc()
        # second once used to cause InvalidParameterError
        pc()
        self.assertEqual(api.content.get_state(doc), 'published')

    def test_check(self):
        self.request.form.update({
            'action': 'check',
            'selectedType[typeId]': 'Folder',
            'basePath': '/',
            'id': 'foobar'
        })
        cc = content.Creator(self.portal, self.request)
        data = cc()
        self.assertEqual(json.loads(data)['valid'], True)

    def test_upload(self):
        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': '1',
            'chunkSize': 1024,
            'totalSize': 1024 * 5,
            'file': BytesIO('X' * 1024),
            'name': 'foobar.bin'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        self.assertTrue('id' in data)
        self.assertTrue(data['success'])
        self.request.form.update({
            'id': data['id']
        })

        for idx in range(4):
            self.request.form.update({
                'action': 'chunk-upload',
                'chunk': str(idx + 2),
                'file': BytesIO('X' * 1024)
            })
            cc = content.Creator(self.portal, self.request)
            data = json.loads(cc())
            self.assertTrue(data['success'])

        self.assertTrue(data['valid'])
        self.assertTrue('url' in data)

        fileOb = api.content.get(path='/file-repository/foobar.bin')
        self.assertEquals(fileOb.file.data, 'X' * 1024 * 5)
        return fileOb

    def test_update_upload(self):
        fileOb = self.test_upload()
        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': '1',
            'chunkSize': 1024,
            'totalSize': 1024,
            'file': BytesIO('U' * 1024),
            'name': 'foobar2.bin',
            'content': IUUID(fileOb),
            'field': 'file'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        self.assertTrue(data['valid'])
        self.assertTrue('url' in data)
        fileOb = api.content.get(path='/file-repository/foobar.bin')
        self.assertEquals(fileOb.file.data, 'U' * 1024)

    def test_tmp_upload(self):
        fileOb = self.test_upload()
        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': '1',
            'chunkSize': 1024,
            'totalSize': 1024,
            'file': BytesIO('T' * 1024),
            'name': 'foobar2.bin',
            'content': IUUID(fileOb),
            'field': 'tmp_blarg'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        self.assertTrue(data['valid'])
        self.assertTrue('url' in data)
        fileOb = api.content.get(path='/file-repository/foobar.bin')

        # should not change!
        self.assertEquals(fileOb.file.data, 'X' * 1024 * 5)

        # tmp file should be uploaded
        annotations = IAnnotations(fileOb)
        tmp_files = annotations['_tmp_files']
        self.assertTrue('tmp_blarg' in tmp_files)

    def test_create_folders(self):
        self.request.form.update({
            'action': 'create',
            'basePath': 'foo/bar',
            'id': 'foobar',
            'title': 'Foobar',
            'selectedType[id]': 'Document',
            'transitionTo': 'publish'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        ob = api.content.get(path='/foo/bar/foobar')
        self.assertTrue(ob is not None)
        self.assertEquals(
            data['base_url'], 'http://nohost/plone/foo/bar/foobar')
        self.assertEquals(
            api.content.get_state(ob), 'published')

    def test_save_file_saves_custom_field_values(self):

        # add new field
        registry = getUtility(IRegistry)
        fields = deepcopy(registry['castle.file_upload_fields'])
        for f in fields:
            f['required'] = unicode(f['required']).lower()
        fields.append({
            u'name': u'foobar',
            u'label': u'Foobar',
            u'widget': u'text',
            u'required': u'false',
            u'for-file-types': u'*'
        })
        registry['castle.file_upload_fields'] = fields

        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': '1',
            'chunkSize': 1024,
            'totalSize': 1024,
            'file': BytesIO('X' * 1024),
            'name': 'foobar.bin',
            'title': 'Foobar',
            'foobar': 'Some value here'
        })
        cc = content.Creator(self.portal, self.request)
        cc()

        fileOb = api.content.get(path='/file-repository/foobar.bin')
        self.assertEquals(fileOb.file.data, 'X' * 1024)
        self.assertEquals(fileOb.foobar, 'Some value here')
        return fileOb

    def test_content_should_implement_empty_feed_settings_to_prevent_errors(self):
        doc = api.content.create(
            type='Document',
            id='docblah',
            container=self.portal)
        settings = IFeedSettings(doc)
        # should not cause TypeError
        self.assertEquals(settings.feed_types, ())


class TestContentAccess(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        logout()

    def test_accees_denied(self):
        content.Creator(self.portal, self.request)()
        self.assertEquals(self.request.response.getStatus(), 403)

    def test_modify_content_upload_permission(self):
        api.user.create(
            email='foo@bar.com', username='foobar',
            password='foobar', roles=('Member',))
        setRoles(self.portal, 'foobar', (
            'Member', 'Contributor', 'Reviewer'))
        api.user.create(
            email='foo2@bar.com', username='foobar2',
            password='foobar2', roles=('Member',))
        setRoles(self.portal, 'foobar2', (
            'Member', 'Reviewer'))

        login(self.portal, 'foobar')
        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': '1',
            'chunkSize': 1024,
            'totalSize': 1024,
            'file': BytesIO('X' * 1024),
            'name': 'foobar.bin'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        fileOb = api.content.get(path='/file-repository/foobar.bin')
        self.assertEquals(fileOb.file.data, 'X' * 1024)

        # and publish the sucker
        api.content.transition(fileOb, 'publish')

        logout()
        login(self.portal, 'foobar2')

        self.request.form.update({
            'action': 'chunk-upload',
            'file': BytesIO('U' * 1024),
            'name': 'foobar2.bin',
            'content': IUUID(fileOb),
            'field': 'file'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        self.assertFalse(data['success'])


class TestPageLayoutSelector(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_get_front_page(self):
        view = content.PageLayoutSelector(
            self.portal['front-page'], self.request)
        result = json.loads(view())
        self.assertEqual(result['success'], True)
        self.assertEqual(result['page_layout'], 'frontpage.html')
        self.assertEqual(result['section_layout'], None)

    def test_set_front_page(self):
        view = content.PageLayoutSelector(
            self.portal['front-page'], self.request)
        self.request.form.update({
            'data': json.dumps({
                'page_layout': 'foobar.html',
                'section_layout': 'foobar.html'
            }),
            'action': 'save'
        })
        view()

        # check results...
        del self.request.form['action']
        result = json.loads(view())
        self.assertEqual(result['success'], True)
        self.assertEqual(result['page_layout'], 'foobar.html')
        self.assertEqual(result['section_layout'], 'foobar.html')

        # also check that the registry entry value set since
        # front page is default page for site
        self.assertEqual(
            self.portal.portal_registry['castle.cms.default_layout'],
            'foobar.html')



# -*- coding: utf-8 -*-
from plone.app.dexterity import _
from plone.app.dexterity.browser import types
from plone.z3cform import layout
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile


class TypeEditForm(types.TypeEditForm):

    # Remove clone button
    def __init__(self, context, request):
        super(TypeEditForm, self).__init__(context, request)
        if 'castlecms-advanced=1' not in request['HTTP_COOKIE']:
            self.buttons = self.buttons.copy().omit('clone')


class TypesListing(types.TypesListing):
    template = ViewPageTemplateFile('templates/types-listing.pt')
    editform_factory = TypeEditForm


# Create a form wrapper so the form gets layout.
TypesListingPage = layout.wrap_form(
    TypesListing, __wrapper_class=types.TypesEditFormWrapper,
    label=_(u'Dexterity Content Types'))



from plone import api
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.resources.browser.cook import cookWhenChangingSettings
import logging
log = logging.getLogger(__name__)

PROFILE_ID = 'profile-castle.cms:2_3_3'


def upgrade(context, logger=None):
    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile(PROFILE_ID)
    # Reindex events that have recurrences
    brains = api.content.find(portal_type='Event')
    for brain in brains:
        obj = brain.getObject()  # unfortunate but necessary
        if hasattr(obj, 'recurrence') and obj.recurrence is not None:
            log.info('reindexing recurrence of %s' % brain.Title)
            obj.reindexObject(idxs=['recurrence'])
    cookWhenChangingSettings(api.portal.get())



from castle.cms.tiles.base import BaseTile
from castle.cms.browser.survey import ICastleSurvey
from zope import schema
from zope.interface import Interface
from Products.CMFPlone.utils import getSiteLogo
from plone.autoform.directives import widget
from z3c.form.browser.select import SelectWidget
from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
import json
from base64 import b64encode


class SurveyTile(BaseTile):

    def getData(self):
        survey_settings = getUtility(IRegistry).forInterface(ICastleSurvey, check=False)
        survey_data = {
            'url': survey_settings.survey_api_url,
            'id': self.data.get('survey_id'),
            'custom_url': self.data.get('survey_url'),
            'rule': self.data.get('rule', 'always'),
            'cookie': self.data.get('use_cookie', 'show_once'),
            'display': self.data.get('display', 'here'),
            'showLogo': self.data.get('show_logo', False),
            'duration': self.data.get('duration', 20),
            'count': self.data.get('page_count', 5),
            'logo': getSiteLogo()
        }
        return b64encode(json.dumps(survey_data))

    def render(self):
        if self.context.portal_membership.isAnonymousUser():
            return "<div class='pat-survey mosaic-survey-tile' data-pat-survey='{data}'/>".format(data=self.getData())  # noqa
        return "<div>Survey Tile: Some visitors will see an invitation.</div>"


class SurveyAPIWidget(SelectWidget):
    def render(self):
        for item in self.items:
            if item['value'] == 'no_api':
                return '<div class="survey-no-api">The CastleCMS Survey API is not configured properly.<br>Please enter a custom URL below, or finish configuring Survey in Site Settings.</div>'  # noqa
        return super(SurveyAPIWidget, self).render()


class ISurveyTileSchema(Interface):
    widget('survey_id', SurveyAPIWidget)
    survey_id = schema.Choice(
       title=u'Surveys',
       description=u'Select a survey from the API:',
       required=False,
       vocabulary='castle.cms.vocabularies.Surveys'
    )

    survey_url = schema.TextLine(
        title=u'Custom Survey URL',
        description=u'Or enter a survey URL manually:',
        required=False
    )

    rule = schema.Choice(
        title=u'Display Rules',
        description=u'When should the survey display?',
        required=True,
        vocabulary=SimpleVocabulary([
            SimpleTerm(u'always', u'always', u'Immediately on each page effected by this tile'),
            SimpleTerm(u'timer', u'timer', u'Countdown/Timer: Visitor has been on this page for some time'),
            SimpleTerm(u'count', u'count', u'Page Count: Visitor has gone to x number of pages'),
            SimpleTerm(u'leave', u'leave', u'When Mouse Leaves top of page')
        ]),
    )

    use_cookie = schema.Choice(
        title=u'Cookies',
        description=u'How should cookies be used to prevent users from seeing the invite repeatedly?',
        required=True,
        vocabulary=SimpleVocabulary([
            SimpleTerm(u'always', u'always', u'Always show the invite based on selected rules'),
            SimpleTerm(u'until_clicked', u'until_clicked', u'Show the invite only until the user clicks it'),
            SimpleTerm(u'show_once', u'show_once', u'Show the invite only once per session')
        ])
    )

    duration = schema.Int(
        title=u'Timer Duration',
        description=u'If Countdown/Timer rule selected, enter duration (seconds).',
        required=False
    )

    page_count = schema.Int(
        title=u'Page Count',
        description=u'If Page Count rule selected, enter number of pages',
        required=False
    )

    display = schema.Choice(
        title=u'Display Mode',
        description=u'Where should the survey display?',
        required=True,
        vocabulary=SimpleVocabulary([
            SimpleTerm(u'modal', u'modal', u'As a Modal overlaying the page.'),
            SimpleTerm(u'here', u'here', u'In the location this tile is placed.')
        ])
    )

    show_logo = schema.Bool(
        title=u'Show site logo in survey invite',
        required=False,
        default=False
    )



from zope.component import queryUtility
from zope.security.interfaces import IPermission
from plone import api


def get_managers():
    managers = []
    for admin_user in api.user.get_users():
        user_roles = api.user.get_roles(user=admin_user)
        admin_email = admin_user.getProperty('email')
        if (('Manager' not in user_roles and
                'Site Administrator' not in user_roles) or
                not admin_email):
            continue
        managers.append(admin_user)
    return managers


def get_permission_title(permission):
    add_perm_ob = queryUtility(IPermission, name=permission)
    if add_perm_ob:
        permission = add_perm_ob.title
    return permission


def publish_content(obj):
    try:
        api.content.transition(obj=obj, transition='publish')
    except api.exc.InvalidParameterError:
        try:
            api.content.transition(obj=obj, transition='publish_internally')
        except api.exc.InvalidParameterError:
            # not a valid transition, move on I guess...
            pass


def is_backend(request):
    backend_urls = api.portal.get_registry_record('plone.backend_url', default=[])
    try:
        for backend_url in backend_urls:
            if backend_url.startswith(request.SERVER_URL):
                return True
    except TypeError:
        return False  # noqa Falsy None from get_registry_record seems to be getting assigned to backend_urls even with default empty list..



from Products.CMFCore.utils import getToolByName

import logging
log = logging.getLogger(__name__)

PROFILE_ID = 'profile-castle.cms:2_3_5'


def upgrade(context, logger=None):
    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile(PROFILE_ID)



import logging
import os
import types
from urllib import unquote

import transaction
from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from castle.cms import cache
from castle.cms.constants import MAX_PASTE_ITEMS
from castle.cms.interfaces import IHasDefaultImage
from castle.cms.interfaces import IReferenceNamedImage
from OFS.CopySupport import CopyError
from OFS.CopySupport import _cb_decode
from OFS.CopySupport import eInvalid
from plone import api
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.contentlisting.interfaces import IContentListingObject
from plone.app.uuid.utils import uuidToCatalogBrain
from plone.dexterity.interfaces import IDexterityContent
from plone.dexterity.interfaces import IDexterityEditForm
from plone.subrequest import subrequest
from ZODB.POSException import POSKeyError


logger = logging.getLogger('castle.cms')


def get_paste_data(req):
    try:
        op, mdatas = _cb_decode(req['__cp'])
    except Exception:
        raise CopyError(eInvalid)

    try:
        if mdatas[0][0].startswith('cache:'):
            cache_key = mdatas[0][0].replace('cache:', '')
            mdatas = cache.get(cache_key)
    except IndexError:
        pass

    paths = []
    for mdata in mdatas:
        paths.append('/'.join(mdata))

    catalog = api.portal.get_tool('portal_catalog')
    count = len(catalog(path={'query': paths, 'depth': -1}))

    return {
        'paths': paths,
        'op': op,
        'mdatas': mdatas,
        'count': count
    }


def is_max_paste_items(paste_data):
    return paste_data['count'] > MAX_PASTE_ITEMS


def recursive_create_path(site, path):
    if path == '/':
        folder = site
    else:
        path = path.lstrip('/')
        folder = site.restrictedTraverse(path, None)
        if folder is None:
            # Need to create folders up to where we want content
            # we'll walk it up create folders as needed
            folder = site
            for part in path.split('/'):
                try:
                    folder = folder[part]
                except KeyError:
                    folder = api.content.create(
                        type='Folder',
                        id=part,
                        title=part.capitalize().replace('-', ' '),
                        container=folder)
    return folder


def clear_object_cache(ob):
    ob._p_jar.invalidateCache()
    transaction.begin()
    ob._p_jar.sync()


def inline_images_in_dom(dom, portal=None, site_url=None):
    if portal is None:
        portal = api.portal.get()
    if site_url is None:
        site_url = portal.absolute_url()
    for img in dom.cssselect('img'):
        src = img.attrib.get('src', '')
        if src.startswith(site_url):
            ctype, data = get_data_from_url(src, portal, site_url)
            if not ctype or not data or 'image' not in ctype:
                img.attrib['src'] = ''
                continue
            data = data.encode("base64").replace("\n", "")
            data_uri = 'data:{0};base64,{1}'.format(ctype, data)
            img.attrib['src'] = data_uri


def get_data_from_url(url, portal=None, site_url=None):
    if portal is None:
        portal = api.portal.get()
    if site_url is None:
        site_url = portal.absolute_url()
    data = None
    path = url.replace(site_url, '').strip('/')
    ct = ''
    if '/@@images/' in path:
        try:
            path, _, im_info = path.partition('/@@images/')
            parts = im_info.split('/')
            if len(parts) == 2:
                fieldname = parts[0]
                size = parts[1]
                images = portal.restrictedTraverse(
                    str(path + '/@@images'), None)
                if images is not None:
                    im = images.traverse(fieldname, [size])
                    try:
                        data = im.scale.data.data
                        ct = im.scale.data.contentType
                    except AttributeError:
                        pass
            else:
                # grab full size
                ob = portal.restrictedTraverse(str(path), None)
                data = ob.image.data
                ct = ob.image.contentType
        except Exception:
            logger.error('Could not traverse image ' + url, exc_info=True)

    if data is None:
        ob = portal.restrictedTraverse(str(path), None)
        file_path = getattr(ob, 'path', None)
        if file_path is None:
            try:
                file_path = ob.context.path
            except Exception:
                pass
        if file_path and os.path.exists(file_path):
            fi = open(file_path)
            data = fi.read()
            fi.close()
            ct = 'image/' + file_path.split('.')[-1].lower()
        else:
            resp = subrequest(unquote(url))
            if resp.status != 200:
                return None, None
            try:
                ct, encoding = resp.getHeader(
                    'content-type').split('charset=')
                ct = ct.split(';')[0]
                # pisa only likes ascii css
                data = resp.getBody().decode(encoding)
            except ValueError:
                ct = resp.getHeader('content-type').split(';')[0]
                data = resp.getBody()

    return ct, data


def is_mosaic_edit_form(request):
    try:
        if (ILayoutAware.providedBy(request.PUBLISHED.context) and
                IDexterityEditForm.providedBy(request.PUBLISHED) and
                request.PUBLISHED.context.getLayout() == 'layout_view' and
                request.PUBLISHED.__name__ == 'edit'):
            return True
    except Exception:
        pass
    return False


def get_object_version(obj, version):
    context = aq_inner(obj)
    if version == "current":
        return context
    else:
        repo_tool = api.portal.get_tool('portal_repository')
        return repo_tool.retrieve(context, int(version)).object


def get_path(obj, site=None):
    """
    Get the full path FROM the site of an object
    path object: /Castle/foo/bar -> /foo/bar
    """
    if site is None:
        site = api.portal.get()
    site_path = site.getPhysicalPath()
    obj_path = obj.getPhysicalPath()
    path = '/' + '/'.join(obj_path[len(site_path):])
    return path


def get_image_info(brain):
    image_info = None
    if IContentListingObject.providedBy(brain):
        brain = brain._brain
    if IDexterityContent.providedBy(brain):
        obj = brain
        try:
            image = obj.image
            if IReferenceNamedImage.providedBy(image):
                uid = image.reference
                brain = uuidToCatalogBrain(uid)
                if brain:
                    return get_image_info(brain)
                else:
                    return
            width, height = image.getImageSize()
            image_info = {
                'width': width,
                'height': height
            }
            try:
                image_info['focal_point'] = image.focal_point
            except Exception:
                try:
                    image_info['focal_point'] = obj._image_focal_point
                except Exception:
                    image_info['focal_point'] = [width / 2, height / 2]
        except AttributeError:
            pass
    else:
        try:
            image_info = brain.image_info
            if 'reference' in image_info:
                uid = image_info['reference']
                brain = uuidToCatalogBrain(uid)
                if brain:
                    return get_image_info(brain)
        except Exception:
            pass

    return image_info


def get_folder_contents(folder, **query):
    query.update({
        'sort_on': 'getObjPositionInParent',
        'path': {
            'query': '/'.join(folder.getPhysicalPath()),
            'depth': 1
        },
        'show_inactive': False
    })
    catalog = api.portal.get_tool('portal_catalog')
    return catalog(**query)


def get_context_from_request(request):
    published = request.get('PUBLISHED')
    if isinstance(published, types.MethodType):
        return published.im_self
    return aq_parent(published)


def has_image(obj):
    try:
        # check if brain has the data
        return obj.hasImage
    except Exception:
        if getattr(obj, 'getObject', False):
            obj = obj.getObject()
        if IHasDefaultImage.providedBy(obj):
            return True
        try:
            return getattr(aq_base(obj), 'image', None) is not None
        except POSKeyError:
            return False



from zope.interface import Interface


class IArchiveContentTransformer(Interface):
    def __init__(archiver):
        pass

    def __call__(dom):
        pass


class IArchiveManager(Interface):
    pass
